proc shade {orig dest frac} {
    if {$frac >= 1.0} { return $dest } elseif {$frac <= 0.0} { return $orig }
    foreach {origR origG origB} [rgb2dec $orig] \
	    {destR destG destB} [rgb2dec $dest] {
	set shade [format "\#%02x%02x%02x" \
		[expr {int($origR+double($destR-$origR)*$frac)}] \
		[expr {int($origG+double($destG-$origG)*$frac)}] \
		[expr {int($origB+double($destB-$origB)*$frac)}]]
	return $shade
    }
}

proc rgb2dec c {
    set c [string tolower $c]
    if {[regexp -nocase {^#([0-9a-f])([0-9a-f])([0-9a-f])$} $c x r g b]} {
	# double'ing the value make #9fc == #99ffcc
	scan "$r$r $g$g $b$b" "%x %x %x" r g b
    } else {
	if {![regexp {^#([0-9a-f]+)$} $c junk hex] || \
		[set len [string length $hex]]>12 || $len%3 != 0} {
	    if {[catch {winfo rgb . $c} rgb]} {
		return -code error "bad color value \"$c\""
	    } else {
		return $rgb
	    }
	}
	set len [expr {$len/3}]
    	scan $hex "%${len}x%${len}x%${len}x" r g b
    }
    return [list $r $g $b]
}

proc xentry {w args} {
  global GUI font

  entry $w -font $font(entry) -selectbackground $GUI(selectbackground) -selectforeground $GUI(selectforeground) -background $GUI(entry_bg) -foreground $GUI(entry_fg)
  eval $w configure $args


  bind $w <Button-3> "Entry_Menu %W"
  bind $w <Button-2> "Entry_Menu %W"
  clipboard clear -displayof $w

}

proc Entry_Menu {w} {

 set x [winfo pointerx .]
 set y [winfo pointery .]

 set m ".mentry[lindex [split $w .] end]"
 destroy $m

 xmenu $m
 if {[catch {selection get -displayof $w}]} {
   set copy "disabled"
 } else {
   set copy "normal"
 }

 if {[$w get]==""} {
   set all "disabled"
 } else {
   set all "normal"
 }

 $m add command -label Copy -command "Entry_Copy $w" -state $copy
 $m add command -label Paste -command "tk_textPaste $w"
 $m add command -label Cut -command "Entry_Cut $w" -state $copy
 $m add separator
 $m add command -label "Select All" -command "$w selection range 0 end" -state $all

 tk_popup $m $x $y
  
}

proc Entry_Cut {w} {
  Entry_Copy $w
  catch {$w delete sel.first sel.last} 
}

proc Entry_Copy {w} {
  catch {
   clipboard clear -displayof $w
   clipboard append -displayof $w [selection get -displayof $w]
  }
}


proc xmclistbox {w args} {
 global GUI font
   set lb_cmd "mclistbox $w \
	    -bd 0 \
	    -columnrelief flat \
	    -labelanchor w \
            -labelbg $GUI(widgets_color) \
	    -columnborderwidth 0 \
	    -labelfont [list $font(listbox)] \
	    -selectmode browse \
	    -background $GUI(listbox_bg) \
            -foreground $GUI(listbox_fg) \
	    -selectforeground $GUI(selectforeground) \
	    -selectbackground $GUI(selectbackground) \
	    -labelborderwidth 2\
            $args"
   eval "$lb_cmd"
}

proc xgrid {w c r cs rs s} {
  eval "grid config $w -column $c -row $r -columnspan $cs -rowspan $rs -sticky $s"
}

proc xscrollbar {w args} {
  global GUI
  if {[::platform::isPlatformUnix]} {
    set sb_cmd "cscrollbar $w -bg lightblue $args -slidercolor $GUI(scrollbar_slider) -gradient1 $GUI(scrollbar_gradient1) -gradient2 $GUI(scrollbar_gradient2)"
  }
  if {[::platform::isPlatformWindows]} {
    set sb_cmd "scrollbar $w -relief flat -bd 0 $args"
  }
  eval "$sb_cmd"
}


proc scrolled_frame {w contents} {
    global GUI

    canvas ${w}_scrollbox -yscrollcommand [list ${w}_vs set] -bd 0 -bg $GUI(widgets_color)
    #-height $h -width $width 
    frame  $w
    xscrollbar ${w}_vs -command [list ${w}_scrollbox yview]

    grid ${w}_scrollbox ${w}_vs -sticky snew

    grid rowconfigure [winfo parent $w] 0 -weight 1
    grid columnconfigure [winfo parent $w] 0 -weight 1
    grid columnconfigure [winfo parent $w] 1 -weight 1 -minsize 16

    uplevel $contents
    update idletasks

    ${w}_scrollbox configure -scrollregion [list 0 0 \
	    [winfo reqwidth $w] [winfo reqheight $w]]
    ${w}_scrollbox create window 0 0 -anchor nw -window $w

}

