package require Metawidget

metawidget create Toolbar {
  pack [frame $this.frm -border 2 -relief flat] -side top -fill both -expand 1

  menu $this.menu -tearoff 0
  $this.menu add command -label Top    -command "$this config -side top"
  $this.menu add command -label Bottom -command "$this config -side bottom"
  $this.menu add command -label Left   -command "$this config -side left"
  $this.menu add command -label Right  -command "$this config -side right"

  bind $this.menu <FocusOut> "$this.menu unpost"

  bind Toolbar$this <1>         "$this _click"
  bind Toolbar$this <B1-Motion> "$this _drag %X %Y"
  bind Toolbar$this <3>         "$this.menu post %X %Y; focus $this.menu"

  bind Toolbutton$this <Enter> {%W config -relief raised}
  bind Toolbutton$this <Leave> {%W config -relief flat}

  my wParent [winfo parent $this]

  -side top
  -state normal
} {} -default frm -border 1 -relief raised

metawidget proc Toolbar _click {} {
  my iX1 [expr [winfo rootx [my wParent]]+20]
  my iY1 [expr [winfo rooty [my wParent]]+20]
  my iX2 [expr [my iX1]+[winfo width  [my wParent]]-40]
  my iY2 [expr [my iY1]+[winfo height [my wParent]]-40]
}

metawidget proc Toolbar _drag { iPx iPy } {
 catch {
  if { $iPx < [my iX1] && [my -side] != "left" } {
    -side left
  } elseif { $iPx > [my iX2] && [my -side] != "right" } {
    -side right
  } elseif { $iPy < [my iY1] && [my -side] != "top" } {
    -side top
  } elseif { $iPy > [my iY2] && [my -side] != "bottom" } {
    -side bottom
  }
 }
}

metawidget proc Toolbar _checkItem { sTag } {
  if { ! [myinfo exists _$sTag.wWidget] } {
    error "Toolbar $this: Item $sTag does not exist."
  }
}

metawidget proc Toolbar _redraw {} {
  set sFill  [mkw.decode [my -side] "top x bottom x left y right y"]
  set sInner [mkw.decode [my -side] "top left bottom left left top right top"]
  set sFirst [lindex [pack slaves [my wParent]] 0]

  if { $sFirst == {} } {
    pack $this -side [my -side] -fill $sFill
  } elseif { [lsearch [bindtags $sFirst] "Statusbar"] != -1 } {
    pack $this -side [my -side] -fill $sFill -after $sFirst
  } else {
    pack $this -side [my -side] -fill $sFill -before $sFirst
  }

  foreach sWidget [winfo children $this.frm] {
    if { [winfo class $sWidget] == "Frame" } {
      if { $sFill == "x" } {
        pack $sWidget -side $sInner -fill y -padx 4 -pady 1
      } else {
        pack $sWidget -side $sInner -fill x -padx 1 -pady 4
      }
    } else {
      pack $sWidget -side $sInner -fill both
    }
  }
}

metawidget proc Toolbar _bindTag { sWhat wWidget sBindTag } {
  bindtags $wWidget [mkw.l$sWhat [bindtags $wWidget] $sBindTag]
}

metawidget proc Toolbar _checkButton { sTag {iState {}} } {
  if { $iState == {} } {
    my _$sTag.iState [expr ! [my _$sTag.iState]]
  } else {
    my _$sTag.iState [mkw.complete $iState {on off yes no 1 0}]
  }

  if { [my _$sTag.iState] } {
    [my _$sTag.wWidget] config -relief sunken
    _bindTag shrink [my _$sTag.wWidget] Toolbutton$this
  } else {
    [my _$sTag.wWidget] config -relief flat
    _bindTag extend [my _$sTag.wWidget] Toolbutton$this
  }
}

metawidget proc Toolbar _radioButton { sTag } {
  set wButton $this.frm._$sTag

  foreach sGroupTag [my __[my _$sTag.sGroup]] {
    $this.frm._$sGroupTag config -relief flat
    _bindTag extend $this.frm._$sGroupTag Toolbutton$this
  }

  [my _$sTag.wWidget] config -relief sunken
  _bindTag shrink [my _$sTag.wWidget] Toolbutton$this
}

metawidget proc Toolbar -side { sSide } {
  my -side [mkw.complete $sSide {top bottom left right}]
  _redraw
}

metawidget proc Toolbar -state { sState } {
  my -state [mkw.complete $sState {normal fixed withdrawn}]

  switch [my -state] {
    normal {
      _bindTag extend $this.frm Toolbar$this
      -side [my -side]
    }
    fixed {
      _bindTag shrink $this.frm Toolbar$this
      -side [my -side]
    }
    withdrawn {
      pack forget $this
    }
  }
}

metawidget proc Toolbar add { sType sTag args } {
  if { [myinfo exists _$sTag.wWidget] } {
    error "Toolbar $this: Item $sTag already exists."
  }

  set sType   [mkw.complete $sType {button checkbutton radiobutton separator}]
  set wWidget $this.frm._$sTag

  my _$sTag.sTag    $sTag
  my _$sTag.sType   $sType
  my _$sTag.wWidget $wWidget
  my _$sTag.iState  0

  if { $sType == "separator" } {
    frame $wWidget -border 1 -relief sunken -width 2 -height 2
    _bindTag extend $wWidget Toolbar$this
  } else {
    button $wWidget -border 1 -relief flat -padx 0 -pady 0 -takefocus 0
    _bindTag extend $wWidget Toolbutton$this

    if { $sType == "checkbutton" } {
      bind $wWidget <1> "$this _checkButton $sTag"
    } elseif { $sType == "radiobutton" } {
      bind $wWidget <1> "$this _radioButton $sTag"

      set args [mkw.options $args * {-group *}]
      if { ! [info exists -group] } {
        set -group default
      }

      my _$sTag.sGroup ${-group}
      if { [myinfo exists __${-group}] } {
        my __${-group} [linsert [my __${-group}] end $sTag]
      } else {
        my __${-group} $sTag
      }
    }
  }

  eval itemconf {$sTag} $args

  if { [my -state] != "withdrawn" } {
    _redraw
  }

  return $wWidget
}

metawidget proc Toolbar delete { args } {
  foreach sTag $args {
    if { [my _$sTag.sType] == "radiobutton" } {
      my __[my _$sTag.sGroup] [mkw.lshrink [my __[my _$sTag.sGroup]] $sTag]
    }

    destroy [my _$sTag.wWidget]
    unmy _$sTag.sTag _$sTag.sType _$sTag.wWidget _$sTag.iState _$sTag.sGroup
  }

  _redraw
}

metawidget proc Toolbar itemconf { sTag args } {
  _checkItem $sTag
  eval {$this.frm._$sTag} config $args
}

metawidget proc Toolbar itemcget { sTag sOption } {
  _checkItem $sTag
  $this.frm._$sTag cget $sOption
}

metawidget proc Toolbar set_ { sTag {iState {}} } {
  _checkItem $sTag

  if { [my _$sTag.sType] == "checkbutton" } {
    _checkButton $sTag $iState
  } elseif { [my _$sTag.sType] == "radiobutton" } {
    _radioButton $sTag
  }
}

metawidget proc Toolbar get { sTag } {
  _checkItem $sTag
  return [my _$sTag.iState]
}

metawidget proc Toolbar invoke { sTag {iState {}} } {
  _checkItem $sTag
  set_ $sTag $iState
  $this.frm._$sTag invoke
}

metawidget proc Toolbar menu_ { args } {
  eval $this.menu $args
}

metawidget proc Toolbar names { {sPattern *} } {
  set lTags {}

  foreach sVar [myinfo vars _*.sTag] {
    if { ! [string match $sPattern [my $sVar]] } continue
    lappend lTags [my $sVar]
  }

  return $lTags
}

metawidget command Toolbar _click       _click
metawidget command Toolbar _drag        _drag
metawidget command Toolbar _checkButton _checkButton
metawidget command Toolbar _radioButton _radioButton

metawidget command Toolbar add          add
metawidget command Toolbar delete       delete
metawidget command Toolbar itemconf     itemconf
metawidget command Toolbar itemcget     itemcget
metawidget command Toolbar set          set_
metawidget command Toolbar get          get
metawidget command Toolbar invoke       invoke
metawidget command Toolbar menu         menu_
metawidget command Toolbar names        names

metawidget option Toolbar -side  -side
metawidget option Toolbar -state -state

