package provide Metawidget 1.3

namespace eval ::metawidget {
  proc metawidget { sCmd {sClass {}} args } {
    set sCommand [mkw.complete $sCmd {create delete command option proc names info export}]
    set iArgc [llength $args]

    switch $sCommand {
      create {
        if { $iArgc < 2 } {
          error "Wrong # args: Must be metawidget create class initbody destroybody ?options?"
        } elseif { ! [string match {[A-Z]*} $sClass] } {
          error "Bad metawidget class name '$sClass'. Class name must start with uppercase character"
        }

        set sConstr [lindex $args 0]
        set sDestr  [lindex $args 1]
        set lArgs   [lrange $args 2 end]

        _createClass $sClass $sConstr $sDestr $lArgs

        variable lMetawidgets
        set lMetawidgets [mkw.lextend $lMetawidgets $sClass]
      }

      delete {
        if { $iArgc > 0 } {
          error "Wrong # args: Must be metawidget delete class"
        }

        _checkClass $sClass
        namespace delete ::$sClass

        variable lMetawidgets
        set lMetawidgets [mkw.lshrink $lMetawidgets $sClass]
      }

      proc {
        if { $iArgc != 3 } {
          error "Wrong # args: Must be metawidget proc class name args body"
        }

        _checkClass $sClass
        mkw.lassign $args sProc lArgs sBody

        if { [lsearch [list $sClass ~$sClass __dispatch __configure __cget] $sProc] != -1 } {
          error "Bad proc name '$sProc': This name is predefined or reserved."
        }

        proc ::${sClass}::$sProc $lArgs "
          upvar this this
          $sBody
        "
      }

      command {
        if { $iArgc != 2 } {
          error "Wrong # args: Must be metawidget command class name proc"
        }

        _checkClass $sClass
        mkw.lassign $args sCommand sProc

        set ::${sClass}::__aCmd($sCommand) $sProc
      }

      option {
        if { $iArgc < 1 || $iArgc > 3 } {
          error "Wrong # args: Must be metawidget option class name ?setProc? ?getProc?"
        }

        _checkClass $sClass
        mkw.lassign $args sOption sSetProc sGetProc

        switch $iArgc {
          1 { set ::${sClass}::__aOpt($sOption) {} }
          2 { set ::${sClass}::__aOpt($sOption) [list $sSetProc] }
          3 { set ::${sClass}::__aOpt($sOption) [list $sSetProc $sGetProc] }
        }
      }

      names {
        if { $iArgc } {
          error "Wrong # args: Must be metawidget names"
        }

        variable lMetawidgets
        return $lMetawidgets
      }

      info {
        if { $iArgc != 1 } {
          error "Wrong # args: Must be metawidget info name option"
        }

        _checkClass $sClass
        _getInfo $sClass [lindex $args 0]
      }

      export {
        if { $iArgc != 0 } {
          error "Wrong # args: Must be metawidget export name"
        }

        _checkClass $sClass

        set sCmd "namespace eval ::$sClass {\n"

        append sCmd "  array set __aOpt \[list [array get ::${sClass}::__aOpt]\]\n"
        append sCmd "  array set __aCmd \[list [array get ::${sClass}::__aCmd]\]\n\n"

        foreach sProc [lsort [info procs ::${sClass}::*]] {
          append sCmd "  proc [namespace tail $sProc] { "
          foreach sArg [info args $sProc] {
            if { [info default $sProc $sArg sDefault] } {
              append sCmd "{$sArg {$sDefault}} "
            } else {
              append sCmd "$sArg "
            }
          }
          append sCmd "} {[info body $sProc]}\n\n"
        }

        append sCmd "  namespace export $sClass\n}\n\n"

        append sCmd "namespace eval :: {
          namespace import ::${sClass}::$sClass
          rename $sClass [string tolower $sClass]
        }"
      }
    }
  }

  proc _checkClass { sClass } {
    if { [info command ::${sClass}::$sClass] == "" } {
      error "Metawidget $sClass does not exist."
    }
  }

  proc _createClass { sClass sConstr sDestr lArgs } {
    set lArgs [mkw.options $lArgs * {-default *} {-type {frame toplevel} frame} {-command *}]

    set sType [string tolower ${-type}]

    if { [info exists -command] } {
      set sCommand ${-command}
    } else {
      set sCommand [string tolower $sClass]
    }

    if { [info exists -default] } {
      set wDefault \$this.${-default}
    } else {
      set wDefault \${this}::\$this
    }

    namespace eval ::$sClass "
      proc $sClass { this args } {
        eval {$sType} {\$this} -class {$sClass} $lArgs

        bindtags \$this \[lreplace \[bindtags \$this\] 1 1 $sClass\]

        rename ::\$this ::\${this}::\$this

        namespace eval :: \"
          proc \$this { {sCmd {}} args } {
            ::${sClass}::__dispatch \$this \\\$sCmd \\\$args
          }
        \"

        $sConstr

        eval __configure \$args

        return \$this
      }

      proc ~$sClass { this } {
        $sDestr
        namespace delete ::\$this
      }

      proc __dispatch { this sCmd lArgs } {
        if { \$sCmd == {} } {
          error \"Wrong # args: Should be '\$this command args ?args ...?'\"
        }

        variable __aCmd

        if { \[info exists __aCmd(\$sCmd)\] } {
          eval {\$__aCmd(\$sCmd)} \$lArgs
        } else {
          set sCommand \[array names __aCmd \${sCmd}*\]
          set iMatches \[llength \$sCommand\]

          if { \$iMatches == 1 } {
            eval {\$__aCmd(\$sCommand)} \$lArgs    ;# successfully resolved
          } elseif { \$iMatches == 0 } {
            eval {$wDefault} {\$sCmd} \$lArgs      ;# not found, try default
          } else {
            error \"Ambiguous command '\$sCmd': Must be \[join \[array names __aCmd\] {, }\]\"
          }
        }
      }

      proc __configure { args } {
        upvar this this
        variable __aOpt

        set iArgs \[llength \$args\]

        if { \$iArgs == 0 } {                        ;# return all options
          return \"Function not yet implemented\"
        } elseif { \$iArgs == 1 } {                  ;# return data for option
          return \"Function not yet implemented\"
        } elseif { \[expr \$iArgs % 2\] } {          ;# odd number of args
          error \"Failed to configure \$this: Invalid number of arguments\"
        } else {                                     ;# process options
          foreach { sOpt sValue } \$args {
            if { \[info exists __aOpt(\$sOpt)\] } {  ;# found in array
              set sOption \$sOpt
            } else {                                 ;# not found, resolve it
              set sOption \[array names __aOpt \${sOpt}*\]
              set iMatches \[llength \$sOption\]

              if { \$iMatches == 0 } {               ;# not found, try default
                $wDefault configure \$sOpt \$sValue
                continue
              } elseif { \$iMatches > 1 } {
                error \"Ambiguous option '\$sOpt': Must be \[join \[array names __aOpt\] {, }\]\"
              }
            }

            if { \[llength \$__aOpt(\$sOption)\] == 0 } { ;# just set variable
              my \$sOpt \$sValue
            } else {                                      ;# call procedure
              eval \[lindex \$__aOpt(\$sOption) 0\] {\$sValue}
            }
          }
        }
      }

      proc __cget { sOpt } {
        upvar this this
        variable __aOpt

        if { \[catch {llength \$__aOpt(\$sOpt)} iLen\] } {
          $wDefault cget \$sOpt
        } elseif { \$iLen != 2 } {
          my \$sOpt
        } else {
          eval \[lindex \$__aOpt(\$sOpt) 1\]
        }
      }

      bind $sClass <Destroy> {::${sClass}::~$sClass %W}

      set __aCmd(configure) __configure
      set __aCmd(cget)      __cget

      namespace export $sClass
      namespace import ::metawidget::un* ::metawidget::my* ::metawidget::our* ::metawidget::mkw.*
    "
    namespace eval :: "
      namespace import ::${sClass}::$sClass
      rename $sClass $sCommand
    "
  }

  proc _getInfo { sClass sOption } {
    switch [mkw.complete $sOption {procs commands options}] {
      procs {
        set lProcs {}
        foreach sProc [info procs ::${sClass}::*] {
          lappend lProcs [list [namespace tail $sProc] [info args $sProc] [info body $sProc]]
        }

        return [lsort -index 0 $lProcs]
      }

      commands {
        set lCmds {}
        foreach {sCmd sProc} [array get ::${sClass}::__aCmd] {
          lappend lCmds [list $sCmd $sProc]
        }

        return [lsort -index 0 $lCmds]
      }

      options {
        set lOpts {}
        foreach {sOpt lProcs} [array get ::${sClass}::__aOpt] {
          lappend lOpts [list $sOpt $lProcs]
        }

        return [lsort -index 0 $lOpts]
      }
    }
  }

  proc my { args } {
    upvar this this
    namespace eval ::$this set $args
  }

  proc unmy { args } {
    upvar this this
    namespace eval ::$this "catch {unset $args}"
  }

  proc myarray { args } {
    upvar this this
    namespace eval ::$this array $args
  }

  proc myinfo { sOption args } {
    upvar this this
    set sClass [lindex [bindtags $this] 1]

    switch $sOption {
      class {
        set sClass
      }

      exists {
        namespace eval ::$this info exists $args
      }

      vars {
        set lVars {}
        if { [llength $args] } {
          set sPattern [lindex $args 0]
        } else {
          set sPattern *
        }

        foreach sVar [info vars ::${this}::$sPattern] {
          lappend lVars [namespace tail $sVar]
        }

        set lVars
      }

      default {
        error "Bad option '$sOption'. Must be class, exists, vars"
      }
    }
  }

  proc our { args } {
    upvar this this

    set sClass [lindex [bindtags $this] 1]
    namespace eval ::$sClass set $args
  }

  proc unour { args } {
    upvar this this
    set sClass [lindex [bindtags $this] 1]
    namespace eval ::$sClass "catch {unset $args}"
  }

  proc ourarray { args } {
    upvar this this
    set sClass [lindex [bindtags $this] 1]
    namespace eval ::$sClass array $args
  }


  proc ourinfo { sOption args } {
    upvar this this
    set sClass [lindex [bindtags $this] 1]

    switch $sOption {
      exists {
        namespace eval ::$sClass info exists $args
      }

      vars {
        set lVars {}
        if { [llength $args] } {
          set sPattern [lindex $args 0]
        } else {
          set sPattern *
        }

        foreach sVar [info vars ::${sClass}::$sPattern] {
          lappend lVars [namespace tail $sVar]
        }

        set lVars
      }

      default {
        error "Bad option '$sOption'. Must be exists, vars"
      }
    }
  }

  proc mkw.lextend { lList args } {
    foreach sArg $args {
      if { [lsearch $lList $sArg] != -1 } continue
      lappend lList $sArg
    }

    return $lList
  }

  proc mkw.lshrink { lList args } {
    foreach sArg $args {
      set iPos [lsearch $lList $sArg]
      if { $iPos == -1 } continue
      set lList [lreplace $lList $iPos $iPos]
    }

    return $lList
  }

  proc mkw.lchange { lList sOld sNew } {
    set iPos [lsearch $lList $sOld]
    if { $iPos != -1 } {
      set lList [lreplace $lList $iPos $iPos $sNew]
    }

    return $lList
  }

  proc mkw.lassign { lList args } {
    set iMax [llength $lList]
    set iCnt 0

    foreach sElem $lList sVar $args {
      if { $sVar == {} } break
      uplevel [list set $sVar $sElem]

      if { [incr iCnt] >= $iMax } break
    }
  }

  proc mkw.decode { sValue lPairs args } {
    foreach { sMatch sRet } $lPairs {
      if { $sMatch != $sValue } continue
      return $sRet
    }

    if { [llength $args] } {
      return [lindex $args 0]
    } else {
      return $sValue
    }
  }

  proc mkw.complete { sShort lValues {sWhat option}} {
    foreach sValue $lValues {
      set aTmp($sValue) {}
    }

    if { [info exists aTmp($sShort)] } {
      return $sShort
    } else {
      set sLong [array names aTmp $sShort*]

      if { [llength $sLong] == 1 } {
        return $sLong
      } elseif { [llength $sLong] == 0 } {
        error "Bad $sWhat '$sShort': Must be [join $lValues ,]"
      } else {
        error "Ambiguous $sWhat '$sShort': Must be [join $lValues ,]"
      }
    }
  }

  proc mkw.options { lArgs args } {
    foreach lSpec $args {
      set iArgc [llength $lSpec]

      if { $iArgc < 1 || $iArgc > 3 } {
        error "Bad option spec '$lSpec': Must be name ?valuelist or *? ?defaultValue?"
      }

      set sOpt [lindex $lSpec 0]
      set aOpt($sOpt) $iArgc
      set aVal($sOpt) [lindex $lSpec 1]

      if { $iArgc == 3 } {
        uplevel [list set $sOpt [lindex $lSpec 2]]
      }
    }

    set lRest  {}            ;# remaining arguments not covered by option specs
    set iArgc  [llength $lArgs]
    set bAllow [info exists aOpt(*)]   ;# true if command shall not complain

    for { set i 0 } { $i < $iArgc } { incr i } {
      set sOpt [lindex $lArgs $i]

      if { [info exists aOpt($sOpt)] } {
        set sOption $sOpt
      } else {
        set sOption [array names aOpt $sOpt*]
      }

      if { [llength $sOption] == 1 } {
        if { $aOpt($sOption) == 1 } {
          uplevel [list set $sOption 1]
        } elseif { $aVal($sOpt) == "*" } {
          incr i
          uplevel [list set $sOption [lindex $lArgs $i]]
        } else {
          incr i
          uplevel [list set $sOption [mkw.complete [lindex $lArgs $i] $aVal($sOpt) "option value"]]
        }
      } elseif { [llength $sOption] > 1 } {
        error "Ambiguous option '$sOpt': Must be [join [array names aOpt] ,]"
      } elseif { ! $bAllow } {
        error "Bad option '$sOpt': Must be [join [array names aOpt] ,]"
      } else {
        lappend lRest $sOpt
      }
    }

    return $lRest
  }

  set lMetawidgets {}

  namespace export metawidget mkw.* un* my* our*
}

namespace import ::metawidget::metawidget

