package provide platform 0.0.1

namespace eval platform {
  namespace export isPlatformUnix
  namespace export isPlatformWindows
  namespace export userHomeDir
  namespace export savircDir
  namespace export tempDir
}

# EASY PLATFORM DETERMINATION PROCS
proc platform::isPlatformUnix {} {
  global tcl_platform
  if { $::tcl_platform(platform) == "unix" } {
    return "true"
  }
  return "false"
}
  
proc platform::isPlatformWindows {} {
  global tcl_platform
  if { $::tcl_platform(platform) == "windows" } {
    return "true"
  }
  return "false"
}
 
# FILESYSTEM ABSTRACTION PROCS
proc platform::fileSeparator {} {
  if {[isPlatformUnix]} {
    return "/"
  }
  if {[isPlatformWindows]} {
    return "\\"
  }
}
  
proc platform::pathSeparator {} {
  if {[isPlatformUnix]} {
    return ":"
  }
  if {[isPlatformWindows]} {
    return ";"
  }
}
  
proc platform::userHomeDir {} {
  global env
  if {[isPlatformUnix]} {
    return "$env(HOME)"
  }
  if {[isPlatformWindows]} {
    if {[file exists "$env(APPDATA)"]} {
      return "$env(APPDATA)"
    } else {
      return "C:\\"
    }
  }
}
  
proc platform::savircDir {} {
  global env
  set savirc_home ""
  catch {set savirc_home $env(SAVIRC_HOME)}
  if {[file exists "$savirc_home"]} {
    return "$savirc_home"
  }
  if {[isPlatformWindows]} {
    return "[file join [userHomeDir] savIRC]"
  } else {
    return "[file join [userHomeDir] .savirc]"
  }
}

proc platform::tempDir {} {
  global env
  if {[isPlatformUnix]} {
    return "/tmp"
  }
  if {[isPlatformWindows]} {
    return $env(TEMP)
  }
}
