
proc config {lbl} {
  global im_ok im_cancel font icons
  destroy .conf
  set w ".conf"
  
  toplevel $w -class Config
  wm transient $w .
  wm title $w [mc "savIRC Configuration"]
  
  
  set width [winfo reqwidth .]
  set height [winfo reqheight .]
  set x [expr { ( [winfo vrootwidth  .] - $width  ) / 2 }]
  set y [expr { ( [winfo vrootheight .] - $height ) / 2 }]
  
  wm geometry $w +${x}+${y}
  

  frame $w.f -width 500
  frame $w.bottom -bd 2
  xscrollbar $w.xscroll -orient horizontal -command "$w.tree xview"
  Tree:create $w.tree -bd 2 -relief sunken -width 130 -xscrollcommand "$w.xscroll set"

  $w.tree bind x <1> {
   set lbl [Tree:labelat %W %x %y]
   catch {OpenConfigTree %W $lbl} e
  }

  grid config $w.tree -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky "snew"
  grid config $w.xscroll -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky "new"
  grid config $w.f -column 1 -row 0 -columnspan 1 -rowspan 3 -sticky "snew"
  grid config $w.bottom -column 0 -row 2 -columnspan 1 -sticky snew

  grid columnconfigure $w 0 -weight 0
  grid columnconfigure $w 1 -weight 1
  grid rowconfigure $w 0 -weight 1
  grid rowconfigure $w 1 -weight 0

  autoscroll $w.xscroll

  xbutton $w.bottom.save -image $icons(savesmall) -side left -text [mc Save] -command "SaveSettings" -font $font(button)
  xbutton $w.bottom.ok -image $im_ok -side left -text [mc Ok] -command "SaveSettings; destroy $w" -font $font(button)
  xbutton $w.bottom.cancel -image $im_cancel -side left -text [mc Cancel] -command "destroy $w;ReLoadSettings" -font $font(button)

  pack $w.bottom.ok $w.bottom.save $w.bottom.cancel -side left -padx 2 -pady 5

  #catch {grab $w}
  FillTree $w.tree
  focus $w

  if {$lbl==""} {
     set lbl [mc /Configuration/Connection/User&Identd]
  } 
  Tree:open $w.tree [file dirname $lbl]
  OpenConfigTree $w.tree $lbl
  bind $w <Return> "SaveSettings; destroy $w"
  bind $w <Escape> "destroy $w; LoadSettings"
  bind $w <KeyPress> "destroy .coltip"
}

proc OpenConfigTree {w lbl} {
   global Tree
   CleanFrame .conf.f $w
   if {[catch {"Conf_[file tail $lbl]" .conf.f}]} {
     if {[info exists Tree($w:$lbl:open)]} {
         if {$Tree($w:$lbl:open)} {
           Tree:close $w $lbl
         } else {
           Tree:open $w $lbl
           switch [file tail $lbl] {
             "GUI" {catch {Conf_ColorMessages .conf.f}}
             "Connection" {catch {Conf_User&Identd .conf.f}}
             "IRC" {catch {Conf_IRC-switches .conf.f}}
           }
         }
     }
   } else {
     Tree:setselection $w $lbl
   }
}

proc FillTree {w} {
   global icons im_conset im_connect im_settings im_gui
   Tree:newitem $w /[mc Configuration] -image i_folder

   Tree:newitem $w [mc /Configuration/Connection] -image $im_connect
   Tree:newitem $w [mc /Configuration/IRC] -image $im_settings
   Tree:newitem $w [mc /Configuration/GUI] -image $im_gui
   Tree:newitem $w [mc /Configuration/Dcc] -image $icons(dccwin)

   Tree:newitem $w [mc /Configuration/Connection/User&Identd] -image $im_conset
   Tree:newitem $w [mc /Configuration/Connection/Server] -image $icons(links)

   Tree:newitem $w [mc /Configuration/IRC/IRC-switches] -image $icons(switches)
   Tree:newitem $w [mc /Configuration/IRC/Messages] -image $icons(text)
   Tree:newitem $w [mc /Configuration/IRC/Logs&Ignore] -image $icons(edit)
   Tree:newitem $w [mc /Configuration/IRC/Files&Folders] -image $icons(addfile)

   Tree:newitem $w [mc /Configuration/GUI/ColorMessages] -image $icons(query)
   Tree:newitem $w [mc /Configuration/GUI/Fonts] -image $icons(font)
   Tree:newitem $w [mc /Configuration/GUI/mIRC-colors] -image $icons(mirccol)
   Tree:newitem $w [mc /Configuration/GUI/Themes] -image $icons(paint)
   Tree:newitem $w [mc /Configuration/GUI/Windows&Tray] -image $icons(cascade)
   Tree:newitem $w [mc /Configuration/GUI/WidgetsColors] -image $icons(widgets)

   Tree:open $w /[mc Configuration]
}


proc CleanFrame {w W} {
   foreach c [winfo children $w] {
      destroy $c
   }
}

proc Conf_Server {w} {
   global GUI me font misc icons DCC

   tabcontrol $w.nb -width auto
   set nb $w.nb

   frame $nb.f0
   frame $nb.f1

   $nb insert servers 0 -text [mc "IRC Servers"] -window $nb.f0
   $nb insert socks 0 -text [mc "SOCKS5 Servers"] -window $nb.f1

   $w.nb invoke servers   
   pack $w.nb -expand yes -fill both -padx 10 -pady 10

   set f [LabelFrame:create $nb.f0.f -text [mc Servers]: -ipadx 10 -ipady 10 -width 400 -height 200]
   pack $nb.f0.f -fill both -expand yes -padx 5 -pady 5
   
   xmclistbox $f.server \
            -labelbg $GUI(widgets_color) \
            -width 100 -height 100 -fillcolumn host -yscrollcommand [list $f.scrollserver set] \
	    -bd 0 -columnrelief flat -labelanchor w -columnborderwidth 0 \
	    -selectmode browse -selectforeground $GUI(selectforeground) \
	    -selectbackground $GUI(selectbackground) -labelborderwidth 1 \
	    -foreground black -background white
   pack $f.server -fill both -expand yes -padx 0 -pady 0 -side left

   $f.server column add host -label [mc Host] -width 8
   $f.server column add port -label [mc Port] -width 6
   $f.server column add desc -label [mc Description] -width 16
   $f.server label bind host <ButtonPress-1> "sort %W host"
   $f.server label bind port <ButtonPress-1> "sort %W port"
   $f.server label bind desc <ButtonPress-1> "sort %W desc"

   xscrollbar $f.scrollserver -command [list $f.server yview]
   
   combobox::combobox $f.net -editable 0 -width 20 -textvariable me(network) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) -command "FillServer $f;"
   
   frame $f.bar
   xbutton $f.bar.add -image $icons(select) -side left -text Add -command "AddServerWin $f" -font $font(button)
   xbutton $f.bar.delete -image $icons(delete) -side left -text Delete -command "DeleteServer $f" -font $font(button)

   checkbutton $f.bar.connect -text [mc "Connect on Startup"] -variable misc(connect)

   grid rowconfigure $f 1 -weight 1
   grid columnconfigure $f 0 -weight 1
   
   xgrid $f.net 0 0 2 1 snew
   xgrid $f.server 0 1 1 1 snew
   autoscroll $f.scrollserver "-in $f -column 1 -row 1 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky nesw"
   xgrid $f.bar 0 2 2 1 snew
   $w.nb invoke servers   
  
   pack $f.bar.add -side left
   pack $f.bar.delete -side left -padx 4
   pack $f.bar.connect -side left -padx 4

   FillNetwork $f.net
   DefaultServer $f

   set f [LabelFrame:create $nb.f1.f -text [mc "SOCKS5 server list:"] -ipadx 10 -ipady 10]
   pack $nb.f1.f -fill both -expand yes -padx 5 -pady 5
   
   xmclistbox $f.socks \
            -labelbg $GUI(widgets_color) \
            -fillcolumn host -yscrollcommand [list $f.scroll set] \
	    -bd 0 -columnrelief flat -labelanchor w -columnborderwidth 0 \
	    -selectmode browse -selectforeground $GUI(selectforeground) \
	    -selectbackground $GUI(selectbackground) -labelborderwidth 1 \
	    -foreground black -background white

   $f.socks column add host -label [mc Host] -width 8
   $f.socks column add port -label [mc Port] -width 5
   $f.socks column add user -label [mc Username] -width 9
   $f.socks column add pass -label [mc Password] -width 9

   xscrollbar $f.scroll -command [list $f.socks yview] 

   frame $f.bar
   xbutton $f.bar.add -image $icons(select) -side left -text [mc Add] -command "AddSockWin $f"  -font $font(button)
   xbutton $f.bar.delete -image $icons(delete) -side left -text [mc Delete] -command "DelSock $f" -font $font(button)
   
   frame $f.checks
   label $f.checks.l -text [mc "Use SOCKS5 for:"]
   checkbutton $f.checks.usesocks  -text [mc Server] -variable me(sock5use)
   checkbutton $f.checks.dccsocks  -text [mc DCC] -variable DCC(sock5)
   pack $f.checks.l $f.checks.usesocks $f.checks.dccsocks -side left
   
   grid rowconfigure $f 1 -weight 1
   grid columnconfigure $f 0 -weight 1
   
   xgrid $f.checks 0 0 2 1 snew
   xgrid $f.socks 0 1 1 1 snew
   autoscroll $f.scroll "-in $f -column 1 -row 1 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky nesw"
   xgrid $f.bar 0 2 2 1 snew

   pack $f.bar.add -side left
   pack $f.bar.delete -side left -padx 4
   #pack $f.bar.usesocks -padx 4
   
   FillSock $f
   DefaultSock $f
  
   $w.nb invoke servers   
}

proc Conf_User&Identd {w} {
   global me GUI misc icons font

   set f [LabelFrame:create $w.f -text [mc "User Settings:"] -ipadx 10 -ipady 12 -width 400 -height 200]

   label $f.lnick -text [mc "Nick Name:"]
   label $f.lalt  -text [mc Alternate:]
   label $f.lreal -text [mc "Real Name:"]
   entry $f.nick -width 25 -textvariable me(nick) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   entry $f.alt  -width 25 -textvariable me(altnick) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   entry $f.real -width 25 -textvariable me(realname) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

   bind $f.real <Control-k> "$f.real insert insert \x03; ColorTip %X %Y ; Preview $f.real; break"
   bind $f.real <Control-b> "$f.real insert insert ; Preview $f.real; break"
   bind $f.real <Control-u> "$f.real insert insert ; Preview $f.real; break"
   bind $f.real <KeyPress> "Preview $f.real"

   bind $f.real <Return> {.status.msg delete 1.0 end }
   bind $f.real <Destroy> ".status.msg delete 1.0 end"

   checkbutton $f.invisible  -text [mc "Invisible Mode"] -variable ""

   label $f.lprofile -text [mc Profile:]
   frame $f.prof

   if {[llength $me(sockslist)]==0} {
     set state "normal"
   } else {
     set state "disabled"
   }
   combobox::combobox $f.prof.profile -state $state -editable 0 -width 10 -textvariable me(profile) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) -command "LoadProfile $f.prof.profile"
   xbutton $f.prof.add -image $icons(select) -state $state -side left -text [mc Add] -width 3 -command "AddProfileWin $f.prof.profile $w"  -font $font(button)
   xbutton $f.prof.del -image $icons(delete) -state $state -side left -text [mc Delete] -wdith 6 -command "DeleteProfile $f.prof.profile $w" -font $font(button)

   pack $f.prof.profile $f.prof.add $f.prof.del -side left

   grid $f.lprofile -column 0 -row 0 -sticky ne -pady 1
   grid $f.prof -column 1 -row 0 -sticky new -pady 1

   grid $f.lnick -column 0 -row 1 -sticky ne -pady 1
   grid $f.nick -column 1 -row 1 -sticky new -pady 1
   grid $f.lalt -column 0 -row 2 -sticky ne -pady 1
   grid $f.alt -column 1 -row 2 -sticky new -pady 1
   grid $f.lreal -column 0 -row 3 -sticky ne -pady 1
   grid $f.real -column 1 -row 3 -sticky new -pady 1
   grid $f.invisible -column 1 -row 4 -sticky nw -pady 1

   pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top

   set f1 [LabelFrame:create $w.f2 -text [mc "Identd Server:"] -ipadx 10 -ipady 11 -width 400 -height 200]

   label $f1.luser -text [mc "User ID:"]
   entry $f1.user  -textvariable me(userid) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   label $f1.lsystem -text [mc System]:
   entry $f1.system  -textvariable me(ident_system) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   label $f1.lport -text [mc Port]:
   entry $f1.port  -textvariable me(ident_port) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

   checkbutton $f1.enable   -command Check_Ident  -text [mc "Enable Identd Server"] -variable me(ident_use)
   checkbutton $f1.connect  -text [mc "Asynchronous Server"] -variable me(ident_connect)
   checkbutton $f1.show     -text [mc "Show Identd Requests"] -variable me(ident_show)

   grid $f1.luser   -column 0 -row 1 -sticky ne -pady 1
   grid $f1.user    -column 1 -row 1 -sticky new -pady 1
   grid $f1.lsystem -column 0 -row 2 -sticky ne -pady 1
   grid $f1.system  -column 1 -row 2 -sticky new -pady 1
   grid $f1.lport   -column 0 -row 3 -sticky ne -pady 1
   grid $f1.port    -column 1 -row 3 -sticky new -pady 1

   grid $f1.enable  -column 1 -columnspan 1 -row 4 -sticky nw 
   grid $f1.connect -column 1 -columnspan 1 -row 5 -sticky nw 
   grid $f1.show   -column 1 -columnspan 1 -row 6 -sticky nw 

   pack $w.f2 -fill both -expand yes -padx 5 -pady 5 -side bottom
   PutProfiles $f.prof.profile
   UpdateProfilesFile
} 

proc PutProfiles {w} {
   global me
   $w list delete 0 end
   foreach p $me(profiles) {
      if {[file exists [file join $me(home) config $p]]} {
        $w list insert end $p
      } else {
        if {$p!="default"} {set me(profiles) [RemoveFromList $me(profiles) $p]}
      }
   }
}


proc DeleteProfile {l w} {
   global me files wid
   if {$me(profile)==""} {return}
   if {[llength $me(sockslist)]!=0} {
      tk_messageBox -message [mc "You can't add/delete or change profile while connected to a server."] -parent $w
      return
   }
   if {$me(profile)=="default"} {
      tk_messageBox -message [mc "You can't delete the default profile."]
      return
   }

   set r [tk_messageBox -type yesno -message [mc "Delete current profile?"] -parent $w]
   if {$r=="no"} {return}
   set file [file join $me(home) config $me(profile)]
   catch {close $wid(winsettings)}
   catch {file delete -force $file}   
   
   set me(profiles) [RemoveFromList $me(profiles) $me(profile)]
   set me(profile) "default"
   PutProfiles $l
   UpdateProfilesFile 
   LoadProfile $l
}


proc LoadProfile {l} {
   global me files GUI
   set oldTheme $GUI(theme)
   
   if {$me(profile)==""} {set me(profile) "default"}
   set files(settings) [file join $me(home) config $me(profile) $me(profile).ini]
   UpdateProfilesFile
   LoadSettings
   set me(command) $me(nick)
   ApplyTheme x x 0

   set files(notify) [file join $me(home) config $me(profile) notify.ini]
   set files(ignore) [file join $me(home) config $me(profile) ignore.ini]
   set files(channels) [file join $me(home) config $me(profile) channels.ini]
   set files(history) [file join $me(home) config $me(profile) history.ini]
   set files(winfile) [file join $me(home) config $me(profile) winfile.ini]

   file mkdir [file join $me(home) config $me(profile) Downloads]
   file mkdir [file join $me(home) config $me(profile) scripts]
   file mkdir [file join $me(home) config $me(profile) logs]
   file mkdir [file join $me(home) config $me(profile) logs Channels]
   file mkdir [file join $me(home) config $me(profile) logs Queries]
   file mkdir [file join $me(home) config $me(profile) logs Chats]

   AddNotifyNicks .main.status.fr.tab.notify.notify.f.f.lnotify
   LoadIgnoreList
   LoadNotifyList
   WinSet
   BgimageHow
   SourceScripts "events"
   SourceScripts "functions"
   SourceScripts "variables"
   SourceScripts "popups"
}

proc AddProfileWin {l w} {
  global im_ok im_cancel GUI font me

  if {[llength $me(sockslist)]!=0} {
      tk_messageBox -message [mc "You can't add/delete or change profile while connected to a server."] -parent $w
      return
  }

  set w ".newprofile"
  if {[winfo exists $w]} {return}
  toplevel $w -class Profile
  wm geometry $w +131+100
  wm resizable $w 0 0
  wm title $w [mc "New Profile"]
  wm transient $w .conf

  set f [LabelFrame:create $w.not -text [mc "New Profile:"] -ipadx 10 -ipady 15]

  frame $w.bottom
  label $f.lprof -text [mc "Enter the name of the profile:"]
  entry $f.prof -width 22 -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
  
  xbutton $w.bottom.ok -width 4 -image $im_ok -side left -text [mc Ok] -command "AddProfile $l $w $f.prof" -font $font(button)
  xbutton $w.bottom.cancel -width 8 -image $im_cancel -side left -text [mc Cancel] -command "destroy $w" -font $font(button)

  grid $w.not -column 0 -row 0 -sticky "snew" -padx 10 -pady 10
  xgrid $w.bottom 0 1 1 1 "snew"

  pack $w.bottom.ok $w.bottom.cancel -side left -padx 15 -pady 10

  grid $f.lprof -column 0 -row 0 -sticky "sne" -pady 2
  grid $f.prof -column 0 -row 1 -sticky "snw" -pady 2

  bind $w <Return> "AddProfile $l $w $f.prof"
  bind $w <Escape> "destroy $w"
  catch {grab $w}
  focus $f.prof
}


proc AddProfile {l w e} {
   global me files
   set t [$e get]
   if {$t==""} {return}
   regsub -all { } $t {_} t
   if {$t=="default"} {
      tk_messageBox -message [mc "You cannot create a profile called, default!"] -parent $w
      return
   }   
   if {[lsearch $me(profiles) $t]!=-1} {
      tk_messageBox -message [format [mc "There is already a profile named, %s."] $t] -parent $w
      return
   }

   destroy $w  
   set me(profile) "$t"

   file mkdir [file join $me(home) config $t]
   file mkdir [file join $me(home) config $t Downloads]
   file mkdir [file join $me(home) config $t Downloads]
   file mkdir [file join $me(home) config $t scripts]
   file mkdir [file join $me(home) config $t logs]
   file mkdir [file join $me(home) config $t logs Channels]
   file mkdir [file join $me(home) config $t logs Queries]
   file mkdir [file join $me(home) config $t logs Chats]

   CreateDefaultScriptFile $t "popups_default_file" "popups.tcl"
   CreateDefaultScriptFile $t "functions_default_file" "functions.tcl"

   set file [file join $me(home) config $t $t.ini]
   if {[catch {open $file w+} fd]} {
      catch {close $fd}
      IRC::echo error $fd
      return      
   }
   WriteConfigFile $fd
   lappend me(profiles) $t
   PutProfiles $l
   UpdateProfilesFile
   set files(settings) $file
   LoadProfile $l
}

proc UpdateProfilesFile {} {
   global files me
   if {[catch {open $files(profiles) w+} fd]} {
      catch {close $fd}
      return      
   }
   puts $fd $me(profile)
   puts $fd $me(profiles)
   close $fd

}

proc Check_Ident {} {
   global me
   if {!$me(ident_use)} {
     if {$me(ident_active)} {Identd_Create}
   } else {
     if {!$me(ident_active)} {Identd_Create}
   }
}


proc Conf_Themes {w} {
    global GUI icons misc me

    set f [LabelFrame:create $w.f -text [mc Themes]: -ipadx 10 -ipady 14 -width 400 -height 200]
    
    listbox $f.themes -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

    frame $f.b
    
    label $f.current -text [format [mc "Active Theme: %s"] [file tail $GUI(theme)]]
    xbutton $f.b.apply -text [mc Apply] -width 8 -command "ApplyTheme $f.themes $f.current 1"
    xbutton $f.b.delete -text [mc Delete] -width 8 -command "DeleteTheme $f.themes $f.current"
    xbutton $f.b.new -text [mc New] -width 8 -command "NewTheme $f.themes"
    message $f.info -width 250 -font "Helvetica 12" \
     -text [mc "A theme includes messages colors, widgets background and foreground,\
     windows icons, windows titlebar background and foreground, selection colors and\
     widgets fonts"]

    grid $f.current -column 0 -row 0 -columnspan 2 -sticky w
    grid $f.themes  -column 0 -row 1 -sticky snew
    grid $f.b       -column 1 -row 1 -sticky n
    grid $f.info    -column 0 -row 2 -sticky snw -columnspan 2

    grid $f.b.apply  -column 0 -row 0 -sticky n -padx 2 -pady 2
    grid $f.b.new    -column 0 -row 1 -sticky n -padx 2 -pady 2
    grid $f.b.delete -column 0 -row 2 -sticky n -padx 2 -pady 2

    pack $w.f -fill both -expand yes -padx 5 -pady 5
    FillThemes $f.themes
}

proc CheckEventIcons {} {
    global GUI
    if {[catch {image type $GUI(IconMin_pressed)}]} {
		set GUI(IconMin_pressed) $GUI(IconMin)
    }
    if {[catch {image type $GUI(IconMin_over)}]} {
		set GUI(IconMin_over) $GUI(IconMin)
    }

    if {[catch {image type $GUI(IconMax_pressed)}]} {
		set GUI(IconMax_pressed) $GUI(IconMax)
    }
    if {[catch {image type $GUI(IconMax_over)}]} {
		set GUI(IconMax_over) $GUI(IconMax)
    }


    if {[catch {image type $GUI(IconRestore_pressed)}]} {
		set GUI(IconRestore_pressed) $GUI(IconRestore)
    }
    if {[catch {image type $GUI(IconRestore_over)}]} {
		set GUI(IconRestore_over) $GUI(IconRestore)
    }

    if {[catch {image type $GUI(IconClose_pressed)}]} {
		set GUI(IconClose_pressed) $GUI(IconClose)
    }
    if {[catch {image type $GUI(IconClose_over)}]} {
		set GUI(IconClose_over) $GUI(IconClose)
    }

}

set GUI(event_icon) "normal"

proc Conf_IconButtons {f} {
	global GUI
    switch -glob -- $GUI(event_icon) {
	"n*" {
		set IconMin "IconMin"
		set IconMax "IconMax"
		set IconRestore "IconRestore"
		set IconClose "IconClose"
	}
	"b*" {
		set IconMin "IconMin_pressed"
		set IconMax "IconMax_pressed"
		set IconRestore "IconRestore_pressed"
		set IconClose "IconClose_pressed"
	}
	"m*" {
		set IconMin "IconMin_over"
		set IconMax "IconMax_over"
		set IconRestore "IconRestore_over"
		set IconClose "IconClose_over"
	}

    }


    $f.icon.iconmin config -image $GUI($IconMin) -command "MDIIconConf $IconMin $f.icon.iconmin $f"
    $f.icon.iconmax config -image $GUI($IconMax) -command "MDIIconConf $IconMax $f.icon.iconmax $f"
    $f.icon.iconres config -image $GUI($IconRestore) -command "MDIIconConf $IconRestore $f.icon.iconres $f"
    $f.icon.iconclo config -image $GUI($IconClose) -command "MDIIconConf $IconClose $f.icon.iconclo $f"

    CheckEventIcons
}

proc Conf_Windows&Tray {w} {
    global GUI icons misc

    set f [LabelFrame:create $w.f -text [mc "Multiple Document Interface:"] -ipadx 10 -ipady 4 -width 400 -height 200]

    checkbutton $f.properties -text [mc "Remember Window Properties"] -variable GUI(properties)
    checkbutton $f.tooltips   -text [mc "Show tooltips for window buttons"] -variable GUI(tooltips)
    label $f.titlebar   -text "[mc "Windows titlebar background:"] " -foreground $GUI(titlebar)
    button $f.btitlebar -image $icons(palette) -relief flat -command "SetTitlebarColor titlebar $f.titlebar $w"
    label $f.foreground -text "[mc "Windows titlebar foreground:"] " -foreground $GUI(tforeground)
    button $f.bforeground -image $icons(palette) -relief flat -command "SetTitlebarColor tforeground $f.foreground $w"

    switch -glob -- $GUI(event_icon) {
	"n*" {
		set IconMin "IconMin"
		set IconMax "IconMax"
		set IconRestore "IconRestore"
		set IconClose "IconClose"
	}
	"b*" {
		set IconMin "IconMin_pressed"
		set IconMax "IconMax_pressed"
		set IconRestore "IconRestore_pressed"
		set IconClose "IconClose_pressed"
	}
	"m*" {
		set IconMin "IconMin_over"
		set IconMax "IconMax_over"
		set IconRestore "IconRestore_over"
		set IconClose "IconClose_over"
	}

    }


    frame $f.icon
    label $f.icon.liconmin -text [mc "Minimize Icon"]
    button $f.icon.iconmin
    label $f.icon.liconmax -text [mc "Maximize Icon"]
    button $f.icon.iconmax
    label $f.icon.liconres -text [mc "Restore Icon"]
    button $f.icon.iconres
    label $f.icon.liconclo -text [mc "Close Icon"]
    button $f.icon.iconclo

    button $f.icon.default -text [mc "Default Icons"] -command "MDIDefaultIcon $f.icon.iconmax $f.icon.iconmin $f.icon.iconres $f.icon.iconclo" -bd 2 -font "Helvetica 12"

    Conf_IconButtons $f

    label $f.icon.levent -text [mc "Display icon on"]
    combobox::combobox $f.icon.event -editable 0 -width 15 -textvariable GUI(event_icon) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) -command "Conf_IconButtons $f"

    $f.icon.event list insert end [mc Normal]
    $f.icon.event list insert end [mc "Button Pressed"]
    $f.icon.event list insert end [mc "Mouse Over"]

    grid config $f.icon.levent -column 0 -row 0 -sticky "nw" -columnspan 2 -pady 5
    grid config $f.icon.event  -column 2 -row 0 -sticky "nw" -columnspan 2 -pady 5

    grid config $f.icon.liconmin -column 0 -row 1 -sticky "nw" -pady 2
    grid config $f.icon.iconmin  -column 1 -row 1 -sticky "w" -padx 4 -pady 2
    grid config $f.icon.liconmax -column 2 -row 1 -sticky "nw" -padx 4 -pady 2
    grid config $f.icon.iconmax  -column 3 -row 1 -sticky "w" -pady 2
    grid config $f.icon.liconres -column 0 -row 2 -sticky "nw" -pady 2
    grid config $f.icon.iconres  -column 1 -row 2 -sticky "w" -padx 4 -pady 2
    grid config $f.icon.liconclo -column 2 -row 2 -sticky "nw" -padx 4 -pady 2
    grid config $f.icon.iconclo  -column 3 -row 2 -sticky "w" -pady 2

    grid config $f.icon.default -column 0 -row 3 -sticky "w" -columnspan 4 -pady 6

    grid config $f.properties -column 0 -row 0 -sticky "nw" -columnspan 2 -pady 2
    grid config $f.tooltips   -column 0 -row 1 -sticky "nw" -columnspan 2 -pady 2

    grid $f.titlebar    -column 0 -row 2 -sticky "nw" -pady 1
    grid $f.btitlebar   -column 1 -row 2 -sticky "nw" -pady 1
    grid $f.foreground  -column 0 -row 3 -sticky "nw" -pady 1
    grid $f.bforeground -column 1 -row 3 -sticky "nw" -pady 1
    grid $f.icon -column 0 -row 4 -sticky "nw" -pady 2

    pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top
    
    
    ##
    ## This is commented out so that User's won't report errors with this. Can reinitialize if it gets fixed
    ## before 1.99 is released.
    ##
    
    #set f [LabelFrame:create $w.f2 -text [mc Tray]: -ipadx 10 -ipady 2 -width 400 -height 200]
    
    #label $f.lab -image $icons(savirc) -relief flat
    #checkbutton $f.use -text [mc "Show savIRC icon in tray"] -variable misc(trayuse) -command TrayCheck
    #checkbutton $f.min -text [mc "savIRC in tray when minimized"] -variable misc(traymin) -command TrayCheck

    #grid config $f.lab -column 0 -row 0 -columnspan 1 -rowspan 2 -sticky "nw" -padx 5
    #xgrid $f.use 1 1 1 1 "nw"
    #xgrid $f.min 1 2 1 1 "nw"

    #pack $w.f2 -fill both -expand yes -padx 5 -pady 5 -side bottom

}

# Unix only
proc TrayCheck {} {
	global misc me GUI

	Tk_Theme
	if {$GUI(tk_theme)!=1} {
		tk_messageBox -message [format [mc "Error loading %s. %s Check that the library was compiled and that theme.so exists in %s!"] $me(path)/Tk_Theme/theme.so \n $me(path)/Tk_Theme]
		set misc(trayuse) 0
		set misc(traymin) 0
	} else {
		if {$misc(trayuse)} {
			catch {PutTray}
		}
	}
}

proc Conf_FServe {w} {
  catch {Conf_FServe2 $w} e
  IRC::echo $e
}

proc ShowTrigger {} {

}

proc Conf_FServe2 {w} {
   global FS icons font GUI me

   set f [LabelFrame:create $w.f -text [mc FServe:] -ipadx 7 -ipady 10 -width 400 -height 200]
   checkbutton $f.enable -text [mc "Enable Fserve:"] -variable me(fserve)
   label $f.lcurrent -text [mc "Current trigger:"]
   combobox::combobox $f.current -editable 0 -width 12 -textvariable FS(current) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)   
   xbutton $f.add -image $icons(select) -side left -text [mc Add] -width 6 -command ""  -font $font(button)
   xbutton $f.del -image $icons(delete) -side left -text [mc Delete] -width 6 -command "" -font $font(button)
   frame $f.trigger   

   grid $f.enable   -column 0 -row 0 -columnspan 4 -sticky nw
   grid $f.lcurrent -column 0 -row 1 -sticky nw
   grid $f.current  -column 1 -row 1 -sticky nw
   grid $f.add -column 2 -row 1 -sticky ne
   grid $f.del -column 3 -row 1 -sticky ne
   grid $f.trigger -column 0 -row 2 -columnspan 4 -sticky snew

   pack $w.f -fill both -expand yes -padx 20 -pady 10 -side top
}

proc Conf_Dcc {w} {
   global DCC GUI files me

   set f [LabelFrame:create $w.f -text "DCC:" -ipadx 7 -ipady 10 -width 400 -height 200]

   checkbutton $f.autoclose_send -text [mc "Autoclose Send"] -variable DCC(autoclose_send) -font "Helvetica 10"
   checkbutton $f.autoclose_get  -text [mc "Autoclose Get"] -variable DCC(autoclose_get) -font "Helvetica 10"
   checkbutton $f.autochat    -text [mc "Autoaccept DCC Chat"] -variable DCC(acceptChat) -font "Helvetica 10"
   checkbutton $f.autosend    -text [mc "Autoaccept DCC Send"] -variable DCC(acceptSend) -command "DCCAutoAccept $f.autosend" -font "Helvetica 10"
   checkbutton $f.autoopen    -text [mc "Open file on success"] -variable DCC(autoopen) -font "Helvetica 10"
   checkbutton $f.ignorec     -text [mc "Ignore DCC Chat"] -variable DCC(ignoreChat) -font "Helvetica 10"
   checkbutton $f.ignores     -text [mc "Ignore DCC Send"] -variable DCC(ignoreSend) -font "Helvetica 10"
   checkbutton $f.minimize    -text [mc "Minimize DCC Send win"] -variable DCC(minimize_send) -font "Helvetica 10"
   checkbutton $f.fservemenu  -text [mc "Use Fserve menu in Chats"] -variable DCC(fserve_menu) -font "Helvetica 10"
   checkbutton $f.passive  -text [mc "Passive DCC requests"] -variable DCC(passive) -font "Helvetica 10"
   label $f.block -text [mc "Packet Size"]

   combobox::combobox $f.eblock -editable 0 -width 4 -textvariable DCC(block) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   $f.eblock list insert end 512
   $f.eblock list insert end 1024
   $f.eblock list insert end 2048
   $f.eblock list insert end 4096

   label $f.lreject -text [mc "Comma separated list of file extensions to reject:"]
   entry $f.reject -textvariable DCC(reject) -width 47 -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

   label $f.incoming -text [mc "Incoming Directory:"]
   xbutton $f.eincoming -width 43 -text "[ShortenEnd $files(incoming) 43]" -command "SetIncoming $f.eincoming"

   grid $f.ignorec  -column 0 -row 0 -sticky "nw" -pady 2
   grid $f.ignores  -column 1 -row 0 -sticky "nw" -pady 2
   grid $f.autochat -column 0 -row 1 -sticky "nw" -pady 2
   grid $f.autosend -column 1 -row 1 -sticky "nw" -pady 2
   grid $f.autoclose_send -column 0 -row 2 -sticky "nw" -pady 2
   grid $f.autoclose_get  -column 1 -row 2 -sticky "nw" -pady 2
   grid $f.autoopen -column 1 -row 3 -sticky "nw" -pady 2
   grid $f.minimize -column 0 -row 3 -sticky "nw" -pady 2
   grid $f.passive  -column 1 -row 4 -sticky "nw" -pady 2
   grid $f.fservemenu -column 0 -row 4 -sticky "nw" -pady 2


   grid $f.block  -column 0 -row 5 -sticky "nw" -pady 5
   grid $f.eblock -column 1 -row 5 -sticky "nw" -pady 5

   grid config $f.incoming -column 0 -row 6 \
	-columnspan 2 -rowspan 1 -sticky "sw" -pady 5

   grid config $f.eincoming -column 0 -row 7 \
	-columnspan 2 -rowspan 1 -sticky "sw" -pady 5

   grid config $f.lreject -column 0 -row 8 \
	-columnspan 2 -rowspan 1 -sticky "sw" -pady 1
   grid config $f.reject -column 0 -row 9 \
	-columnspan 2 -rowspan 1 -sticky "sw" -pady 2

   pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top

}

proc DCCAutoAccept {chk} {
   global DCC GUI icons font im_ok im_cancel
   if {!$DCC(acceptSend)} {return}
   set x [winfo pointerx .]
   set y [winfo pointery .]
   set w ".dccAuAcc"
   toplevel $w -class AuAccept
   wm title $w [mc "DCC AutoAccept"]
   wm geometry $w +$x+$y
   wm resizable $w 0 0
   wm transient $w .conf
   if {$DCC(trust)} {
     set state "normal"
   } else {
     set state "disabled"
   }  
   set f [LabelFrame:create $w.f -text [mc "When AutoAccepting:"] -ipadx 5 -ipady 10 -width 400 -height 200]
   label $f.ifexist -text [mc "If the file already exists:"]
   combobox::combobox $f.exist -editable 0 -width 8 -textvariable DCC(ifexist) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) 
   $f.exist list insert end Ask
   $f.exist list insert end Overwrite
   $f.exist list insert end Resume
   $f.exist list insert end Rename
   $f.exist list insert end Cancel
   checkbutton $f.trust -text [format [mc "Autoget only files from the following %s trusted users masks (e.g. nick!*@*.*):"] \n] -variable DCC(trust) -command "AutoGetState $f.entry"
   frame $f.but
   listbox $f.list -height 4 -bg white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   foreach t $DCC(trusted) {
      $f.list insert end $t
   }
   set trusted $DCC(trusted)
   set trust $DCC(trust)
   set ifexist $DCC(ifexist)
   xbutton $f.but.add -state $state -image $icons(select) -side left -text [mc Add] -width 6 -command "AddAutoAcceptMask $f.entry $f.list"  -font $font(button)
   xbutton $f.but.del -state $state -image $icons(delete) -side left -text [mc Delete] -width 6 -command "DelAutoAcceptMask $f.entry $f.list" -font $font(button)

   entry $f.entry -state $state -bg white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   bind $f.entry <Return> "AddAutoAcceptMask %W $f.list"

   xgrid $f.ifexist 0 0 1 1 nw
   xgrid $f.exist 1 0 1 1 nw
   xgrid $f.trust 0 1 2 1 nw
   xgrid $f.list 0 2 1 1 nw
   xgrid $f.but 1 2 1 1 nw
   xgrid $f.entry 0 3 2 1 nw

   grid $f.but.add -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky ne -padx 5
   grid $f.but.del -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ne -padx 5 -pady 5

   frame $w.bottom -bd 2
   pack $w.f -fill both -expand yes   
   pack $w.bottom -side bottom -fill x
   xbutton $w.bottom.ok -image $im_ok -side left -text [mc Ok] -command "OkAutoGet $f.list; destroy $w" -font $font(button)
   xbutton $w.bottom.cancel -image $im_cancel -side left -text [mc Cancel] -command "destroy $w; DestroyAutoGet $trust $ifexist $trusted" -font $font(button)
   pack $w.bottom.ok $w.bottom.cancel -side left -padx 10 -pady 5
   wm withdraw $w
   wm deiconify $w
   catch {grab $w}
   
}

proc OkAutoGet {l} {
   global DCC
   set DCC(trusted) ""
   set trusted [$l get 0 end]
   foreach t $trusted {
      lappend DCC(trusted) $t
   }
}

proc DestroyAutoGet {trust ifexist args} {
  global DCC
  set DCC(trusted) [join $args]
  set DCC(trust) $trust
  set DCC(ifexist) $ifexist
}

proc AutoGetState {args} {
   global DCC
   if {$DCC(trust)} {
     set state "normal"
   } else {
     set state "disabled"
   }  
   foreach w $args {
      $w config -state $state
   }
}

proc DelAutoAcceptMask {e l} { 
   set s [$l curselection]
   if {$s==""} {return}
   $l delete $s $s
   catch {$l selection set [expr $s-1] [expr $s-1]}
}

proc AddAutoAcceptMask {e l} { 
   set mask [$e get]
   $e delete 0 end
   if {$mask==""} {return}
   if {[string first "!" $mask]==-1&&[string first "@" $mask]==-1} {
      set mask "$mask!*@*.*"
   }
   $l insert 0 $mask
}

proc Conf_mIRC-colors {w} {
    global MIRCcols font

    set f [LabelFrame:create $w.f -text [mc "mIRC Colors:"] -ipadx 10 -ipady 20 -width 400 -height 200]

    checkbutton $f.win  -text [mc "Show Color Window"] -variable MIRCcols(win)
    checkbutton $f.preview  -text [mc "Preview Text"] -variable MIRCcols(preview)

    xbutton $f.default -text [mc "Default mIRC Colors"] -command "DefaultMircColors $f.col"

    set y 0
    for {set x 0; set j 0} {$x < 16} {incr x; incr j} {
      if {$x==8} {set y 1; set j 0}
      label $f.col$x -width 2 -foreground white -background $MIRCcols($x) -text $x
      bind $f.col$x <Button-1> "SetMircColor $x $f.col$x $w"
      grid config $f.col$x -column $j -row $y -padx 1 -pady 1 -columnspan 1 -rowspan 1 -sticky "new"
    }

    xgrid $f.win 0 2 8 1 "snw"
    xgrid $f.preview 0 3 8 1 "snw"
    xgrid $f.default 0 4 8 1 "sw"

    pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top
  
}

proc Conf_Fonts {w} {

    global GUI me font misc icons colsTXT

    set f [LabelFrame:create $w.f -text [mc "Widget Fonts:"] -ipadx 10 -ipady 20 -width 400 -height 200]

    label $f.text    -text [mc "Textbox Font"] -font $font(n)
    label $f.listbox -text [mc "Listbox Font"] -font $font(listbox)
    label $f.label   -text [mc "Label Font"] -font $font(label)
    label $f.entry   -text [mc "Editbox Font"] -font $font(entry)
    label $f.button  -text [mc "Button Font"] -font $font(button)
    label $f.menu    -text [mc "Menu Font"] -font $font(menu)

    button $f.ctext    -image $icons(font) -relief flat -command "SetFont n $f.text"
    button $f.clistbox -image $icons(font) -relief flat -command "SetFont listbox $f.listbox"
    button $f.clabel   -image $icons(font) -relief flat -command "SetFont label  $f.label"
    button $f.centry   -image $icons(font) -relief flat -command "SetFont entry  $f.entry"
    button $f.cbutton  -image $icons(font) -relief flat -command "SetFont button $f.button"
    button $f.cmenu    -image $icons(font) -relief flat -command "SetFont menu $f.menu"

    xgrid $f.text 0 0 1 1 "nw"
    xgrid $f.ctext 1 0 1 1 "nw"
    xgrid $f.listbox 0 1 1 1 "nw"
    xgrid $f.clistbox 1 1 1 1 "nw"
    xgrid $f.label 0 2 1 1 "nw"
    xgrid $f.clabel 1 2 1 1 "nw"
    xgrid $f.entry 0 3 1 1 "nw"
    xgrid $f.centry 1 3 1 1 "nw"
    xgrid $f.button 0 4 1 1 "nw"
    xgrid $f.cbutton 1 4 1 1 "nw"
    xgrid $f.menu 0 5 1 1 "nw"
    xgrid $f.cmenu 1 5 1 1 "nw"

    pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top


    set f [LabelFrame:create $w.f2 -text [mc Selection:] -ipadx 10 -ipady 20 -width 400 -height 200]
     
    label $f.fore -text [mc Foreground] -foreground $GUI(selectforeground)
    label $f.back -text [mc Background] -foreground $GUI(selectbackground)

    button $f.fcol -image $icons(palette) -relief flat -command "SetSelColor selectforeground $f.fore $GUI(selectforeground) $w"
    button $f.bcol -image $icons(palette) -relief flat -command "SetSelColor selectbackground $f.back $GUI(selectbackground) $w"

    xgrid $f.fore 0 0 1 1 "nw"
    xgrid $f.fcol 1 0 1 1 "nw"
    xgrid $f.back 0 1 1 1 "nw"
    xgrid $f.bcol 1 1 1 1 "nw"

    pack $w.f2 -fill x -expand yes -padx 5 -pady 5 -side bottom
}

proc SetWidgetsColor {l w} {
   global GUI
  
   set tmpCOL [tk_chooseColor -initialcolor $GUI(widgets_color) -title [mc "Choose a color:"] -parent $w]
   if {$tmpCOL==""} { return }
   set GUI(widgets_color) $tmpCOL
   $l configure -foreground [shade $GUI(widgets_color) "#000000" 0.2]
   option add *background $GUI(widgets_color) widgetDefault;
   tk_messageBox -message [format [mc "savIRC needs to be restarted for the %s new color to be used by all widgets"] \n]
}


proc SetBackgroundColor {l w} {
   global GUI

   set tmpCOL [tk_chooseColor -initialcolor $GUI(background) -title [mc "Choose a color:"] -parent $w]
   if {$tmpCOL==""} { return }
   set GUI(background) $tmpCOL
   .main config -bg $GUI(background)
   $l configure -foreground $GUI(background)   
}

proc SetBgImage {f w} {
   global GUI files
   set file [tk_getOpenFile -initialdir $files(defaultdir) -title [mc "Choose a background image:"] -parent $w]
   if {$file==""} {return}
   if {[string tolower [file extension $file]]!=".gif"} {
      tk_messageBox -message [mc "Only GIF format is supported."]
      return
   } 
   set GUI(bgimage) $file
   set files(defaultdir) [file dirname $file]
   catch {
    $f.bgimage.t config -text [ShortenEnd $file 30]
    $f.centered config -state normal
    $f.tiled config -state normal
   }
   BgimageHow 
}

proc DelBgImage {f} {
   global GUI
   set GUI(bgimage) ""
   catch {$f.bgimage.t config -text ""}
   catch {image delete bgimage}
   .main delete all

}

proc BgimageHow {} {
  global GUI
  if {$GUI(bgimage)==""} {return}
  if {$GUI(bgimage_how)=="center"} {
    CreateCenterImage
  } else {
    CreateTileImage
  }
}

proc CreateCenterImage {} {
  global GUI
  catch {image delete bgimage}
  if {[catch {image create photo bgimage -file $GUI(bgimage)} bg]} {return}
  .main delete all
  set x [expr [winfo width .]/2]
  set y [expr [winfo height .]/2]
  .main create image $x $y -image $bg    
}

proc CreateTileImage {} {
  global GUI
  catch {image delete bgimage}
  if {[catch {image create photo bgimage -file $GUI(bgimage)} bg]} {return}
  .main delete all
  set h [image height $bg] 
  set w [image width $bg]
  set width [expr [winfo screenwidth .]+$w]
  set height [expr [winfo screenheight .]+$h]

  set row [expr $h/2]
  set col [expr $w/2]
  while {$row<$height} {
     set col [expr $w/2]
     while {$col<$width} {
        .main create image $col $row -image $bg    
        incr col $w
     }
     incr row $h
  }
}

proc ShortenEnd {t n} {
   if {[string length $t]<=$n} {
     return $t
   }
   return "...[string range $t [expr [string length $t]-$n] end]"
}

proc Conf_WidgetsColors {w} {
   global misc icons me font GUI

   set f2 [LabelFrame:create $w.f2 -text [mc "Widget Background:"] -ipadx 10 -ipady 20 -width 400 -height 200]
   
   label $f2.lss  -text "[mc "Scrollbar slider color"]" -foreground $GUI(scrollbar_slider)
   button $f2.bss -image $icons(palette) -relief flat -command "SetGround $f2.lss scrollbar_slider Scrollbar -background $w"

   label $f2.lsg1  -text [mc "Scrollbar gradient"]1 -foreground $GUI(scrollbar_gradient1)
   button $f2.bsg1 -image $icons(palette) -relief flat -command "SetGround $f2.lsg1 scrollbar_gradient1 Scrollbar -background $w"

   label $f2.lsg2  -text [mc "Scrollbar gradient"]2 -foreground $GUI(scrollbar_gradient2)
   button $f2.bsg2 -image $icons(palette) -relief flat -command "SetGround $f2.lsg2 scrollbar_gradient2 Scrollbar -background $w"

   label $f2.lsb  -text [mc "Scrollbar background"] -foreground $GUI(scrollbar_bg)
   button $f2.bsb -image $icons(palette) -relief flat -command "SetGround $f2.lsb scrollbar_bg Scrollbar -background $w"

   label $f2.ldef -text [mc "Default widget color"] -foreground [shade $GUI(widgets_color) "#000000" 0.2]
   button $f2.def -image $icons(palette) -relief flat -command "SetWidgetsColor $f2.ldef $w"

   label $f2.lmain -text [mc "Background color"]: -foreground $GUI(background)
   button $f2.main -image $icons(palette) -relief flat -command "SetBackgroundColor $f2.lmain $w"

   label $f2.btext   -text [mc "Textbox background"] -foreground $GUI(text_bg)
   button $f2.bbtext -image $icons(palette) -relief flat -command "SetGround $f2.btext text_bg Text -background $w"

   label $f2.fentry   -text [mc "Editbox foreground"] -foreground $GUI(entry_fg)
   button $f2.fbentry -image $icons(palette) -relief flat -command "SetGround $f2.fentry entry_fg Entry -foreground $w"
   
   label $f2.bentry   -text [mc "Editbox background"] -foreground $GUI(entry_bg)
   button $f2.bbentry -image $icons(palette) -relief flat -command "SetGround $f2.bentry entry_bg Entry -background $w"

   label $f2.flistbox   -text [mc "Listbox foreground"] -foreground $GUI(listbox_fg)
   button $f2.fblistbox -image $icons(palette) -relief flat -command "SetGround $f2.flistbox listbox_fg Listbox -foreground $w"
   
   label $f2.blistbox   -text [mc "Listbox background"] -foreground $GUI(listbox_bg)
   button $f2.bblistbox -image $icons(palette) -relief flat -command "SetGround $f2.blistbox listbox_bg Listbox -background $w"

   label $f2.lbgimage  -text [mc "Background Image:"]
   
   xbutton $f2.bgimage -text "[ShortenEnd $GUI(bgimage) 36]" -command "SetBgImage $f2 $w" -width 36
   xbutton $f2.delete  -text [mc Delete] -command "DelBgImage $f2"
   
   if {"$GUI(bgimage)"==""||[file exists "$GUI(bgimage)"]!=1} {
     set state "disabled"
   } else {
     set state "normal"
   }
   radiobutton $f2.centered -state $state -text [mc "Center Image"] -variable GUI(bgimage_how) -value "center" -command BgimageHow
   radiobutton $f2.tiled -state $state -text [mc "Tile Image"] -variable GUI(bgimage_how) -value "tile" -command BgimageHow
   
   grid $f2.lmain  -column 0 -row 0 -sticky "snw" -pady 5
   grid $f2.main   -column 1 -row 0 -sticky "snw" -pady 5
   grid $f2.btext  -column 2 -row 0 -sticky "snw" -pady 5
   grid $f2.bbtext -column 3 -row 0 -sticky "snw" -pady 5

   grid $f2.fentry  -column 0 -row 1 -sticky "snw" -pady 5
   grid $f2.fbentry -column 1 -row 1 -sticky "snw" -pady 5
   grid $f2.bentry  -column 2 -row 1 -sticky "snw" -pady 5
   grid $f2.bbentry -column 3 -row 1 -sticky "snw" -pady 5

   grid $f2.flistbox  -column 0 -row 2 -sticky "snw" -pady 5
   grid $f2.fblistbox -column 1 -row 2 -sticky "snw" -pady 5
   grid $f2.blistbox  -column 2 -row 2 -sticky "snw" -pady 5
   grid $f2.bblistbox -column 3 -row 2 -sticky "snw" -pady 5

   grid $f2.lsg1 -column 0 -row 3 -sticky "snw" -pady 5
   grid $f2.bsg1 -column 1 -row 3 -sticky "snw" -pady 5
   grid $f2.lsg2 -column 2 -row 3 -sticky "snw" -pady 5
   grid $f2.bsg2 -column 3 -row 3 -sticky "snw" -pady 5

   grid $f2.lss -column 0 -row 4 -sticky snw -pady 5
   grid $f2.bss -column 1 -row 4 -sticky snw -pady 5

   grid $f2.lsb -column 2 -row 4 -sticky snw -pady 5
   grid $f2.bsb -column 3 -row 4 -sticky snw -pady 5

   grid $f2.ldef -column 0 -row 5 -sticky snw -pady 5
   grid $f2.def  -column 1 -row 5 -sticky snw -pady 5

   grid $f2.lbgimage -column 0 -row 6 -columnspan 4 -sticky snw
   grid $f2.bgimage  -column 0 -row 7 -columnspan 3 -sticky snw
   grid $f2.delete   -column 3 -row 7 -columnspan 1 -sticky snw -padx 0
   grid $f2.centered -column 0 -row 8 -columnspan 2 -sticky snw -pady 5
   grid $f2.tiled    -column 2 -row 8 -columnspan 2 -sticky snw -pady 5
   
   pack $w.f2 -fill both -expand yes -padx 5 -pady 5 -side top
}


proc Conf_ColorMessages {w} {
   global GUI me font misc icons colsTXT

    set f [LabelFrame:create $w.f -text [mc "Color Messages"] -ipadx 10 -ipady 20 -width 400 -height 200]
    #grid $f -sticky nsew
    scrolled_frame $f.scrolledbox {

    set v "$f.scrolledbox"

    label $v.prog -text [mc Program] -foreground $colsTXT(prog)
    label $v.serv -text [mc Server] -foreground $colsTXT(serv)
    label $v.chan -text [mc "Channel text"] -foreground $colsTXT(chan)
    label $v.mechan  -text [mc "Your channel text"] -foreground $colsTXT(mechan)
    label $v.query   -text [mc "Query text"] -foreground $colsTXT(query)
    label $v.mequery -text [mc "Your query text"] -foreground $colsTXT(mequery)
    label $v.chat   -text [mc "Chat text"] -foreground $colsTXT(chat)
    label $v.mechat -text [mc "Your chat text"] -foreground $colsTXT(mechat)
    label $v.notice -text [mc Notice] -foreground $colsTXT(notice)
    label $v.ctcp   -text "CTCP" -foreground $colsTXT(ctcp)
    label $v.whois  -text "Whois" -foreground $colsTXT(whois)
    label $v.invite -text "Invite" -foreground $colsTXT(invite)
    label $v.join  -text "Join" -foreground $colsTXT(join)
    label $v.part  -text "Part" -foreground $colsTXT(part)
    label $v.quit  -text "Quit" -foreground $colsTXT(quit)
    label $v.mode  -text [mc "Mode changes"] -foreground $colsTXT(mode)
    label $v.topic -text [mc "Topic changes"] -foreground $colsTXT(topic)
    label $v.kick  -text "Kick" -foreground $colsTXT(kick)
    label $v.ban   -text "Ban" -foreground $colsTXT(ban)
    label $v.action -text [mc "Channel action"] -foreground $colsTXT(action)
    label $v.nick -text [mc "Nick changes"] -foreground $colsTXT(nick)
    label $v.raw  -text [mc "Raw replies"] -foreground $colsTXT(raw)
    label $v.motd -text "MOTD" -foreground $colsTXT(motd)
    label $v.notify -text [mc Notify] -foreground $colsTXT(notify)
    label $v.dns -text "DNS" -foreground $colsTXT(dns)

    button $v.cprog -image $icons(palette) -relief flat -command "SetColor prog $v.prog $colsTXT(prog) $w"
    button $v.cserv -image $icons(palette) -relief flat -command "SetColor serv $v.serv $colsTXT(serv) $w"
    button $v.cchan -image $icons(palette) -relief flat -command "SetColor chan $v.chan $colsTXT(chan) $w"
    button $v.cmechan  -image $icons(palette) -relief flat -command "SetColor mechan $v.mechan $colsTXT(mechan) $w"
    button $v.cquery   -image $icons(palette) -relief flat -command "SetColor query $v.query $colsTXT(query) $w"
    button $v.cmequery -image $icons(palette) -relief flat -command "SetColor mequery $v.mequery $colsTXT(mequery) $w"
    button $v.cchat   -image $icons(palette) -relief flat -command "SetColor chat $v.chat $colsTXT(chat) $w"
    button $v.cmechat -image $icons(palette) -relief flat -command "SetColor mechat $v.mechat $colsTXT(mechat) $w"
    button $v.cnotice -image $icons(palette) -relief flat -command "SetColor notice $v.notice $colsTXT(notice) $w"
    button $v.cctcp   -image $icons(palette) -relief flat -command "SetColor ctcp $v.ctcp $colsTXT(ctcp) $w"
    button $v.cwhois  -image $icons(palette) -relief flat -command "SetColor whois $v.whois $colsTXT(whois) $w"
    button $v.cinvite -image $icons(palette) -relief flat -command "SetColor invite $v.invite $colsTXT(invite) $w"
    button $v.cjoin   -image $icons(palette) -relief flat -command "SetColor join $v.join $colsTXT(join) $w"
    button $v.cpart   -image $icons(palette) -relief flat -command "SetColor part $v.part $colsTXT(part) $w"
    button $v.cquit   -image $icons(palette) -relief flat -command "SetColor quit $v.quit $colsTXT(quit) $w"
    button $v.cmode   -image $icons(palette) -relief flat -command "SetColor mode $v.mode $colsTXT(mode) $w"
    button $v.ctopic  -image $icons(palette) -relief flat -command "SetColor topic $v.topic $colsTXT(topic) $w"
    button $v.ckick   -image $icons(palette) -relief flat -command "SetColor kick $v.kick $colsTXT(kick) $w"
    button $v.cban    -image $icons(palette) -relief flat -command "SetColor ban $v.ban $colsTXT(ban) $w"
    button $v.caction -image $icons(palette) -relief flat -command "SetColor action $v.action $colsTXT(action) $w"
    button $v.cnick   -image $icons(palette) -relief flat -command "SetColor nick $v.nick $colsTXT(nick) $w"
    button $v.craw    -image $icons(palette) -relief flat -command "SetColor raw $v.raw $colsTXT(raw) $w"
    button $v.cmotd   -image $icons(palette) -relief flat -command "SetColor motd $v.motd $colsTXT(motd) $w"
    button $v.cnotify -image $icons(palette) -relief flat -command "SetColor notify $v.notify $colsTXT(notify) $w"
    button $v.cdns    -image $icons(palette) -relief flat -command "SetColor dns $v.dns $colsTXT(dns) $w"


   xgrid $v.prog 0 0 1 1 "snw"
   xgrid $v.cprog 1 0 1 1 "snw"
   xgrid $v.serv 0 1 1 1 "snw"
   xgrid $v.cserv 1 1 1 1 "snw"
   xgrid $v.chan 0 2 1 1 "snw"
   xgrid $v.cchan 1 2 1 1 "snw"
   xgrid $v.mechan 0 3 1 1 "snw"
   xgrid $v.cmechan 1 3 1 1 "snw"
   xgrid $v.query 0 4 1 1 "snw"
   xgrid $v.cquery 1 4 1 1 "snw"
   xgrid $v.mequery 0 5 1 1 "snw"
   xgrid $v.cmequery 1 5 1 1 "snw"
   xgrid $v.chat 0 6 1 1 "snw"
   xgrid $v.cchat 1 6 1 1 "snw"
   xgrid $v.mechat 0 7 1 1 "snw"
   xgrid $v.cmechat 1 7 1 1 "snw"
   xgrid $v.notice 0 8 1 1 "snw"
   xgrid $v.cnotice 1 8 1 1 "snw"
   xgrid $v.ctcp 0 9 1 1 "snw"
   xgrid $v.cctcp 1 9 1 1 "snw"
   xgrid $v.whois 0 10 1 1 "snw"
   xgrid $v.cwhois 1 10 1 1 "snw"
   xgrid $v.invite 0 11 1 1 "snw"
   xgrid $v.cinvite 1 11 1 1 "snw"
   xgrid $v.join 0 12 1 1 "snw"
   xgrid $v.cjoin 1 12 1 1 "snw"
   xgrid $v.part 0 13 1 1 "snw"
   xgrid $v.cpart 1 13 1 1 "snw"
   xgrid $v.quit 0 14 1 1 "snw"
   xgrid $v.cquit 1 14 1 1 "snw"
   xgrid $v.mode 0 15 1 1 "snw"
   xgrid $v.cmode 1 15 1 1 "snw"
   xgrid $v.topic 0 16 1 1 "snw"
   xgrid $v.ctopic 1 16 1 1 "snw"
   xgrid $v.kick 0 17 1 1 "snw"
   xgrid $v.ckick 1 17 1 1 "snw"
   xgrid $v.ban 0 18 1 1 "snw"
   xgrid $v.cban 1 18 1 1 "snw"
   xgrid $v.action 0 19 1 1 "snw"
   xgrid $v.caction 1 19 1 1 "snw"
   xgrid $v.nick 0 20 1 1 "snw"
   xgrid $v.cnick 1 20 1 1 "snw"
   xgrid $v.raw 0 21 1 1 "snw"
   xgrid $v.craw 1 21 1 1 "snw"
   xgrid $v.motd 0 22 1 1 "snw"
   xgrid $v.cmotd 1 22 1 1 "snw"
   xgrid $v.notify 0 23 1 1 "snw"
   xgrid $v.cnotify 1 23 1 1 "snw"
   xgrid $v.dns 0 24 1 1 "snw"
   xgrid $v.cdns 1 24 1 1 "snw"


   }
   
   pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top

}


proc Conf_IRC-switches {w} {
   global GUI me font misc icons colsTXT 

   set f [LabelFrame:create $w.f -text [mc "IRC Switches:"] -ipadx 10 -ipady 20 -width 400 -height 200]

   scrolled_frame $f.s {

   checkbutton $f.s.join 	-text [mc "Join channel on invite"] -variable misc(join)
   checkbutton $f.s.invite 	-text [mc "Popup msgbox on invite"] -variable misc(invite)
   checkbutton $f.s.rejoin 	-text [mc "Rejoin channel on kick"] -variable misc(rejoin)
   checkbutton $f.s.showtopic 	-text [mc "Show topic in channel"] -variable misc(showtopic)
   checkbutton $f.s.ncompl 	-text [mc "Nick completion (TAB)"] -variable misc(ncompl)
   checkbutton $f.s.ping	-text [mc "Show server pings"] -variable misc(ping)
   checkbutton $f.s.timestamp	-text [mc "Timestamp events"] -variable misc(timestamp)
   checkbutton $f.s.icons	-text [mc "Show icons for events"] -variable icons(show)
   checkbutton $f.s.short	-text [mc "Short Join/Part/Quit"] -variable misc(short)
   checkbutton $f.s.address	-text [mc "Show Addresses"] -variable misc(address)
   checkbutton $f.s.blimit	-text [mc "Limit Buffer"] -variable misc(buffer_limit)
   
   spinentry $f.s.slimit -width 8 -onerror "return" -background white -minimum 50 -step 5 -speed 100 -textvariable misc(buffer_lines) -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) 
   label $f.s.llimit -text [mc "To lines:"]

   checkbutton $f.s.history1  -text [mc "Autocomplete Text"] -variable misc(history)
   checkbutton $f.s.history2  -text [mc "Autocomplete Commands"] -variable misc(historycmd)
   checkbutton $f.s.highlight -text [mc "Highlight Nick in Channels"] -variable misc(highlight)

   frame $f.s.highlightfr
   
   label $f.s.highlightfr.lhighlight -text [mc "Hightlight with color:"]
   button $f.s.highlightfr.bhighlight -image $icons(palette) -relief flat -command "SetHighlightC $w"
   
   pack $f.s.highlightfr.lhighlight -side left
   pack $f.s.highlightfr.bhighlight -side left -padx 5

   frame $f.s.completion
   
   entry $f.s.completion.e -textvariable misc(completion) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) -width 7 
   label $f.s.completion.l -text [mc "nick compl. style:"]
   
   pack $f.s.completion.l $f.s.completion.e -side left

   frame $f.s.fr
   label $f.s.fr.lsplitter -text [mc "Send Splitter:"]
   combobox::combobox $f.s.fr.splitter -editable 0 -width 4 -textvariable misc(splitter) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   $f.s.fr.splitter list insert end 400
   $f.s.fr.splitter list insert end 405
   $f.s.fr.splitter list insert end 410
   $f.s.fr.splitter list insert end 415
   $f.s.fr.splitter list insert end 420
   $f.s.fr.splitter list insert end 425
   $f.s.fr.splitter list insert end 430
   $f.s.fr.splitter list insert end 435
   $f.s.fr.splitter list insert end 440
   $f.s.fr.splitter list insert end 445
   $f.s.fr.splitter list insert end 450
   $f.s.fr.splitter list insert end 455
   $f.s.fr.splitter list insert end 460

   pack $f.s.fr.lsplitter -side left
   pack $f.s.fr.splitter -side left -padx 5



   xgrid $f.s.timestamp 0 0 2 1 "snw"
   xgrid $f.s.ping 0 1 2 1 "snw"
   xgrid $f.s.invite 0 2 2 1 "snw"
   xgrid $f.s.join 0 3 2 1 "snw"
   xgrid $f.s.rejoin 0 4 2 1 "snw"
   xgrid $f.s.icons 0 5 2 1 "snw"
   xgrid $f.s.address 0 6 2 1 "snw"
   xgrid $f.s.short 0 7 2 1 "snw"
   xgrid $f.s.history1 0 8 2 1 "snw"
   xgrid $f.s.history2 0 9 2 1 "snw"
   xgrid $f.s.showtopic 0 10 2 1 "snw"
   xgrid $f.s.blimit 0 11 2 1 "snw"

   xgrid $f.s.llimit 0 12 1 1 "snw"
   xgrid $f.s.slimit 1 12 1 1 "snw"   

   xgrid $f.s.ncompl 0 13 2 1 "snw"
   grid config $f.s.completion -column 0 -row 14 \
	-columnspan 2 -rowspan 1 -sticky "snw" -pady 5
   xgrid $f.s.highlight 0 15 2 1 "snw"
   grid config $f.s.highlightfr -column 0 -row 16 \
	-columnspan 2 -rowspan 1 -sticky "snw" -pady 5
   grid config $f.s.fr -column 0 -row 17 \
	-columnspan 2 -rowspan 1 -sticky "snw" -pady 5

   }

   pack $w.f -fill both -expand yes -padx 5 -pady 5

}


proc Conf_Messages {w} {
    global GUI me font misc icons clientMSG 

    set f [LabelFrame:create $w.f -text [mc "Client Messages:"] -ipadx 10 -ipady 6 -width 400 -height 200]

    label $f.part -text "Part:"
    entry $f.epart -width 30 -textvariable clientMSG(part) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
    
    label $f.quit -text "Quit:"
    entry $f.equit -width 30 -textvariable clientMSG(quit) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
    
    label $f.kick -text "Kick:"
    entry $f.ekick -width 30 -textvariable clientMSG(kick) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
    
    label $f.userinfo -text "Userinfo:"
    entry $f.euserinfo -width 30 -textvariable clientMSG(userinfo) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
    
    label $f.away -text "Away:"
    entry $f.eaway -width 30 -textvariable clientMSG(away) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

    grid config $f.part -column 0 -row 0 \
	-columnspan 1 -rowspan 1 -sticky "sne" -pady 2
    grid config $f.epart -column 1 -row 0 \
	-columnspan 1 -rowspan 1 -sticky "snw" -pady 2
    grid config $f.quit -column 0 -row 1 \
	-columnspan 1 -rowspan 1 -sticky "sne" -pady 2
    grid config $f.equit -column 1 -row 1 \
	-columnspan 1 -rowspan 1 -sticky "snw" -pady 2
    grid config $f.kick -column 0 -row 2 \
	-columnspan 1 -rowspan 1 -sticky "sne" -pady 2
    grid config $f.ekick -column 1 -row 2 \
	-columnspan 1 -rowspan 1 -sticky "snw" -pady 2
    grid config $f.userinfo -column 0 -row 3 \
	-columnspan 1 -rowspan 1 -sticky "sne" -pady 2
    grid config $f.euserinfo -column 1 -row 3 \
	-columnspan 1 -rowspan 1 -sticky "snw" -pady 2
    grid config $f.away -column 0 -row 4 \
	-columnspan 1 -rowspan 1 -sticky "sne" -pady 2
    grid config $f.eaway -column 1 -row 4 \
	-columnspan 1 -rowspan 1 -sticky "snw" -pady 2

    pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top

    set f [LabelFrame:create $w.f2 -text [mc "Show in active window:"] -ipadx 10 -ipady 4 -width 400 -height 200]

    checkbutton $f.whois   -text "Whois" -variable act(whois)
    checkbutton $f.notice  -text [mc Notice] -variable act(notice)
    checkbutton $f.nick    -text [mc "Nick changes"] -variable act(nick)
    checkbutton $f.notify  -text "Notify" -variable act(notify)
    checkbutton $f.invite  -text "Invite" -variable act(invite)
    checkbutton $f.ctcp    -text "CTCP" -variable act(ctcp)

    xgrid $f.whois 0 0 1 1 "nw"
    xgrid $f.notice 1 0 1 1 "nw"
    xgrid $f.nick 0 1 1 1 "nw"
    xgrid $f.notify 1 1 1 1 "nw"
    xgrid $f.invite 0 2 1 1 "nw"
    xgrid $f.ctcp 1 2 1 1 "nw"

    pack $w.f2 -fill both -expand yes -padx 5 -pady 5 -side bottom
}



proc Conf_Logs&Ignore {w} {
    global misc GUI icons font 

    set f [LabelFrame:create $w.f -text [mc "Client Messages:"] -ipadx 10 -ipady 6 -width 400 -height 200]

    frame $f.logs
    checkbutton $f.logs.channel -text [mc "log channels"] -variable misc(logchannel)
    checkbutton $f.logs.query   -text [mc "log query"] -variable misc(logquery)
    checkbutton $f.logs.chat    -text [mc "log chat"] -variable misc(logchat)

    frame $f.bar
    label $f.bar.msg -text [mc "Truncate log when it exceeds Kb:"]
    combobox::combobox $f.bar.trunc -editable 0 -width 4 -textvariable misc(trunclog) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)


    $f.bar.trunc list insert end 50
    $f.bar.trunc list insert end 100
    $f.bar.trunc list insert end 200
    $f.bar.trunc list insert end 500
    $f.bar.trunc list insert end 1000

    checkbutton $f.bar.timestamp  -text [mc "Include timestamps in logs"] -variable misc(logtimestamp)

    set win $f
    canvas $win.flist -background white -height 30 -width 180 -borderwidth 1 -relief sunken -yscrollcommand "$win.fscrolly set"
    flist::Show $win.flist
    xscrollbar $win.fscrolly -command "$win.flist yview"

    xgrid $win.flist 1 0 1 2 nsew
    xgrid $win.fscrolly 2 0 1 2 ns 

    xgrid $f.logs 0 0 1 2 "nw"

    xgrid $f.logs.channel 0 0 1 1 "nw"
    xgrid $f.logs.query 0 1 1 1 "nw"
    xgrid $f.logs.chat 0 2 1 1 "nw"

    grid config $f.bar -column 0 -row 3 -columnspan 2 -rowspan 1 -sticky "nw" -pady 10
    xgrid $f.bar.msg 0 0 1 1 "nw"
    grid config $f.bar.trunc -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky "nw" -padx 6
    grid config $f.bar.timestamp -column 0 -row 1 -sticky nw -columnspan 2

    pack $w.f -fill both -expand yes -padx 5 -pady 5 -side top

    set f [LabelFrame:create $w.f2 -text [mc "Ignore List:"] -ipadx 10 -ipady 4 -width 400 -height 200]


    if {[::platform::isPlatformUnix]} {
      xmclistbox $f.ignore \
            -labelbg $GUI(widgets_color) \
            -height 3 -width 40 -fillcolumn mask -yscrollcommand [list $f.scroll set] \
	    -bd 0 -columnrelief flat -labelanchor w -columnborderwidth 0 \
	    -selectmode browse -selectforeground $GUI(selectforeground) \
	    -selectbackground $GUI(selectbackground) -labelborderwidth 1 -background white
    }
    
    if {[::platform::isPlatformWindows]} {
      mclistbox $f.ignore \
            -labelbg $GUI(widgets_color) \
            -height 3 -width 40 -fillcolumn mask -yscrollcommand [list $f.scroll set] \
	    -bd 0 -columnrelief flat -labelanchor w -columnborderwidth 0 \
	    -selectmode browse -selectforeground $GUI(selectforeground) \
	    -selectbackground $GUI(selectbackground) -labelborderwidth 2 -background white
    }
    $f.ignore column add mask -label [mc Mask] -width 32
    $f.ignore column add flags -label [mc Flags] -width 6

    xscrollbar $f.scroll -command [list $f.ignore yview]

    frame $f.bar
    xbutton $f.bar.add -image $icons(select) -side left -text Add -command "AddIgnoreWin $f" -font $font(button)
    xbutton $f.bar.delete -image $icons(delete) -side left -text Delete -command "DelIgnore $f.ignore" -font $font(button)

    xgrid $f.ignore 0 1 2 1 "snew"
    autoscroll $f.scroll "-in $f -column 2 -row 1 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky nesw"

    pack $f.bar.add -side left
    pack $f.bar.delete -side left -fill x -padx 4

    grid config $f.bar -column 0 -row 2 -columnspan 3 -rowspan 1 -sticky "snew"	-pady 5

    pack $w.f2 -fill both -expand yes -padx 5 -pady 5 -side bottom
    FillIgnore $f
}


# Unix specific
proc Conf_Files&Folders {w} {
   global GUI me font files misc icons colsTXT

   set f [LabelFrame:create $w.f -text [mc "Files & Folders:"] -ipadx 10 -ipady 20 -width 400 -height 200]
   message $f.info -text [format [mc "Set the path for your browser %s that savIRC will use to open URLs %s and the command to open incoming files and the download folder (e.g. your file manager)"] \n \n \n] -width 400 -font "Helvetica 12"

   label $f.blab -text [mc "Browser path:"]
   frame $f.b
   combobox::combobox $f.b.browser -editable 1 -width 30 -textvariable files(browser) -background white -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)
   foreach b [Browsers] {
      $f.b.browser list insert end $b
   }
   button $f.b.choose -text ". . ." -font {-size 10 -family Helvetica} -command "SetBrowser $w" -bd 1
   pack $f.b.browser -side left -expand yes -fill x -padx 2
   pack $f.b.choose -side left
   label $f.dlab -text [mc "Open Downloads folder command:"]
   entry $f.dl -width 36 -bg white -textvariable misc(downloads) -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground) 
   
   set msg [format [mc "You can use the %s and %s identifiers to refer to the downloads folder%s and the incoming file"] %downloads %file \n]
   message $f.msg -text "$msg" -width 300 -font {-family Helvetica -size 11}

   label $f.ldif -text [mc "Open incoming files command:"]
   entry $f.dif -width 36 -bg white -textvariable misc(open_files) -selectforeground $GUI(selectforeground) -selectbackground $GUI(selectbackground)

   grid $f.info -column 0 -row 0 -sticky we
   grid $f.blab -column 0 -row 1 -sticky nw -padx 5
   grid $f.b    -column 0 -row 2 -sticky nw -padx 5
   grid $f.dlab -column 0 -row 3 -sticky nw -padx 5
   grid $f.dl   -column 0 -row 4 -sticky nw -padx 5

   grid $f.ldif -column 0 -row 5 -sticky nw -padx 5 -pady 2
   grid $f.dif  -column 0 -row 6 -sticky nw -padx 5
   grid $f.msg  -column 0 -row 7 -sticky nw -pady 0


   pack $w.f -fill both -expand yes -padx 5 -pady 5
}


