option add *highlightThickness 0

###  set Tree(font) "Helvetica 12"

set Tree(font) \
      -adobe-helvetica-medium-r-normal-*-12-100-100-100-p-76-iso8859-1

proc Tree:create {w args} {
  global Tree
  eval canvas $w -bg white $args
  bind $w <Destroy> "Tree:delitem $w /"
  Tree:dfltconfig $w /
  Tree:buildwhenidle $w
  set Tree($w:selection) {}
  set Tree($w:selidx) {}
}

proc Tree:dfltconfig {w v} {
  global Tree
  set Tree($w:$v:children) {}
  set Tree($w:$v:open) 0
  set Tree($w:$v:icon) {}
  set Tree($w:$v:tags) {}
}

proc Tree:config {w args} {
  eval $w config $args
}

proc Tree:newitem {w v args} {
  global Tree

  set oldv $v  
  regsub -all { } $v {@} v

  set dir [file dirname $v]
  set n [file tail $v]
  if {![info exists Tree($w:$dir:open)]} {
    error "parent item $dir is missing"
  }
  set i [lsearch -exact $Tree($w:$dir:children) $n]
  if {$i>=0} {
    #tk_messageBox -icon error -message "There is already a procedure called \"[file tail $oldv]\"\nprocedure names are unique"    
    return
  }
  lappend Tree($w:$dir:children) $n
  set Tree($w:$dir:children) [lsort $Tree($w:$dir:children)]
  Tree:dfltconfig $w $v
  foreach {op arg} $args {
    switch -exact -- $op {
      -image {set Tree($w:$v:icon) $arg}
      -tags {set Tree($w:$v:tags) $arg}
    }
  }
  Tree:buildwhenidle $w
}

proc Tree:delitem {w v} {
  global Tree
  if {![info exists Tree($w:$v:open)]} return
  if {[string compare $v /]==0} {
    # delete the whole widget
    catch {destroy $w}
    foreach t [array names Tree $w:*] {
      unset Tree($t)
    }
  }
  if {[catch {set boh $Tree($w:$v:children)}]} {return}
  foreach c $Tree($w:$v:children) {
    catch {Tree:delitem $w $v/$c}
  }
  unset Tree($w:$v:open)
  unset Tree($w:$v:children)
  unset Tree($w:$v:icon)
  set dir [file dirname $v]
  set n [file tail $v]
  set i [lsearch -exact $Tree($w:$dir:children) $n]
  if {$i>=0} {
    set Tree($w:$dir:children) [lreplace $Tree($w:$dir:children) $i $i]
  }
  Tree:buildwhenidle $w
}

proc Tree:setselection {w v} {
  global Tree
  regsub -all { } $v {@} v
  set Tree($w:selection) $v
  Tree:drawselection $w
}

proc Tree:getselection w {
  global Tree
  return $Tree($w:selection)
}


image create photo Tree:openbm -data {
	R0lGODlhCQAJANUAAAAAAP///3iYtbDC0+3t5/X18fLy7vDw7Pf39P39+/z8+vf39fb29Pz8
	++rp4/Hw6+zr5uTh2ff289LMv9fSx9bRxtzYz9/b0uPg2ca+rtLMwNvWzNjTycG4p8C3psO6
	qs/Iu+Xh2sK4qP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAACMALAAA
	AAAJAAkAAAY2wIFgSBQIA8hkYNhIOBsKyXCxQCAYj9CwAOh2L0PDgQBxYDbDiIVDqWhAw0lG
	1Ol4PsYicRAEADs=

}

image create photo Tree:closedbm -data {
	R0lGODlhCQAJANUAAAAAAP///3iYtbDC0+3t5/X18fLy7vDw7Pf39P39+/z8+vf39fb29Pz8
	++rp4/Hw6+Th2ff289LMv9bRxtzYz9/b0uPg2ca+rtLMwNvWzNjTycG4p8C3psO6qs/Iu+Xh
	2sK4qP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAACEALAAA
	AAAJAAkAAAY3wIFgSBQIA8hkYNhIJAANRWS4WCAAjMdnWAB4vZWh4UAAOCyZIYSiAUwwnqHk
	AtpsOB1jkTgIAgA7

}


proc Tree:build w {
  global Tree
  $w delete all
  catch {unset Tree($w:buildpending)}
  set Tree($w:y) 30
  Tree:buildlayer $w / 10
  $w config -scrollregion [$w bbox all]
  Tree:drawselection $w
}

proc Tree:buildlayer {w v in} {
  global Tree
  if {$v=="/"} {
    set vx {}
  } else {
    set vx $v
  }
  set start [expr $Tree($w:y)-10]
  foreach c $Tree($w:$v:children) {
    set y $Tree($w:y)
    incr Tree($w:y) 17
    $w create line $in $y [expr $in+10] $y -fill gray50 
    set icon $Tree($w:$vx/$c:icon)
    set taglist x
    foreach tag $Tree($w:$vx/$c:tags) {
      lappend taglist $tag
    }
    set x [expr $in+12]
    if {[string length $icon]>0} {
      set k [$w create image $x $y -image $icon -anchor w -tags $taglist]
      incr x 20
      set Tree($w:tag:$k) $vx/$c
    }
	
    regsub -all {@} $c { } new_txt
    set j [$w create text $x $y -text $new_txt -font $Tree(font) \
                                -anchor w -tags $taglist]
    set Tree($w:tag:$j) $vx/$c
    set Tree($w:$vx/$c:tag) $j
    if {[string length $Tree($w:$vx/$c:children)]} {
      if {$Tree($w:$vx/$c:open)} {
         set j [$w create image $in $y -image Tree:openbm]
         $w bind $j <1> "set Tree($w:$vx/$c:open) 0; Tree:build $w"
         Tree:buildlayer $w $vx/$c [expr $in+18]
      } else {
         set j [$w create image $in $y -image Tree:closedbm]
         $w bind $j <1> "set Tree($w:$vx/$c:open) 1; Tree:build $w"
      }
    }
  }
  if {[catch {$w create line $in $start $in [expr $y+1] -fill gray50} j]} {
    return
  }
  $w lower $j
}

proc Tree:open {w v} {
  global Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==0
      && [info exists Tree($w:$v:children)] 
      && [string length $Tree($w:$v:children)]>0} {
    set Tree($w:$v:open) 1
    Tree:build $w
  }
}

proc Tree:close {w v} {
  global Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==1} {
    set Tree($w:$v:open) 0
    Tree:build $w
  }
}

proc Tree:drawselection w {
  global Tree GUI
  if {[string length $Tree($w:selidx)]} {
    $w delete $Tree($w:selidx)
  }
  set v $Tree($w:selection)
  if {[string length $v]==0} return
  if {![info exists Tree($w:$v:tag)]} return
  set bbox [$w bbox $Tree($w:$v:tag)]
  if {[llength $bbox]==4} {
    set i [eval $w create rectangle $bbox -fill white -outline $GUI(selectbackground)]
    set Tree($w:selidx) $i
    $w lower $i
  } else {
    set Tree($w:selidx) {}
  }
}

proc Tree:buildwhenidle w {
  global Tree
  if {![info exists Tree($w:buildpending)]} {
    set Tree($w:buildpending) 1
    after idle "Tree:build $w"
  }
}

proc Tree:labelat {w x y} {
  set x [$w canvasx $x]
  set y [$w canvasy $y]
  global Tree
  foreach m [$w find overlapping $x $y $x $y] {
    if {[info exists Tree($w:tag:$m)]} {
      set txt $Tree($w:tag:$m)
      regsub -all {@} $txt { } txt
      return $txt
    }
  }
  return ""
}
